// -*- C++ -*-
/*!
 * @file BufferBase.h
 * @brief Buffer abstract class
 * @date $Date: 2007/09/21 09:18:12 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: BufferBase.h,v 1.2.4.2 2007/09/21 09:18:12 n-ando Exp $
 *
 */

/*
 * $Log: BufferBase.h,v $
 * Revision 1.2.4.2  2007/09/21 09:18:12  n-ando
 * Documentation was modified.
 *
 * Revision 1.2  2006/12/02 18:24:15  n-ando
 * BufferBase's interfaces were changed.
 *
 * Revision 1.1  2006/11/27 09:44:34  n-ando
 * The first commitment.
 *
 */

#ifndef BufferBase_h
#define BufferBase_h

#define BUF_SUCCESS 0
#define BUF_TIMEOUT 1
#define BUF_NO_DATA 2 // empty for read
#define BUF_NO_BUF  3 // full for write
#define BUF_FATAL  -1

namespace RTC
{
  /*!
   * @if jp
   * @class BufferBase
   * @brief BufferBase ݥ饹
   * 
   * ΥХåեΤݥ󥿡ե饹
   * ݥХåե饹ϡʲν貾۴ؿμ󶡤ʤФʤʤ
   *
   * public󥿡եȤưʲΤΤ󶡤롣
   * - write(): Хåե˽񤭹
   * - read(): Хåեɤ߽Ф
   * - length(): ХåեĹ֤
   * - isFull(): ХåեդǤ
   * - isEmpty(): ХåեǤ
   *
   * protected󥿡եȤưʲΤΤ󶡤롣
   * - put(): Хåե˥ǡ񤭹
   * - get(): Хåեǡɤ߽Ф
   *
   *
   *
   * @param DataType Хåե˳Ǽǡ
   *
   * @else
   * @class BufferBase
   * @brief BufferBase abstract class
   *
   * This is the abstract interface class for various Buffer.
   *
   * @param DataType Data type to be stored to the buffer.
   *
   * @endif
   */
  template <class DataType>
  class BufferBase
  {
  public:
    /*!
     * @if jp
     *
     * @brief ۥǥȥ饯
     * 
     * @else
     *
     * @brief virtual destractor
     *
     * @endif
     */
    virtual ~BufferBase(){};

    /*!
     * @if jp
     *
     * @brief ХåեĹ
     * 
     * @else
     *
     * @brief Get the buffer length
     *
     * @endif
     */
    virtual long int length() const = 0;

    /*!
     * @if jp
     *
     * @brief Хåե˥ǡ񤭹
     * 
     * @else
     *
     * @brief Write data into the buffer
     *
     * @endif
     */
    virtual int write(const DataType& value) = 0;

    /*!
     * @if jp
     *
     * @brief Хåե˥ǡ񤭹
     * 
     * @else
     *
     * @brief Write data into the buffer
     *
     * @endif
     */
    virtual int read(DataType& value) = 0;

    /*!
     * @if jp
     *
     * @brief ХåեfullǤ
     * 
     * @else
     *
     * @brief True if the buffer is full, else false.
     *
     * @endif
     */
    virtual bool isFull() const = 0;

    /*!
     * @if jp
     *
     * @brief ХåեemptyǤ
     * 
     * @else
     *
     * @brief True if the buffer is empty, else false.
     *
     * @endif
     */
    virtual bool isEmpty() const = 0;

  protected:
    /*!
     * @if jp
     *
     * @brief Хåե˥ǡ񤭹
     * 
     * @else
     *
     * @brief Write data into the buffer
     *
     * @endif
     */
    virtual int put(const DataType& data) = 0;

    /*!
     * @if jp
     *
     * @brief Хåեǡ
     * 
     * @else
     *
     * @brief Get data from the buffer
     *
     * @endif
     */
    virtual int get(DataType& value) = 0;

    /*!
     * @if jp
     *
     * @brief ˽񤭹ХåեλȤ
     * 
     * @else
     *
     * @brief Get the buffer's reference to be written the next
     *
     * @endif
     */
    virtual DataType& getRef() = 0;

  };




  template <class DataType>
  class NullBuffer
    : public BufferBase<DataType>
  {
  public:
    NullBuffer(long int size = 1)
      : m_length(1)
    {
    }

    virtual ~NullBuffer()
    {
    }

    virtual long int length() const
    {
      return 1;
    }

    virtual int write(const DataType& value)
    {
      m_data = value;
      return BUF_SUCCESS;
    }

    virtual int read(DataType& value)
    {
      value = m_data;
      return BUF_SUCCESS;
    }

    virtual bool isFull() const
    {
      return false;
    }

    virtual bool isEmpty() const
    {
      return false;
    }
    
  protected:
    virtual int put(const DataType& data)
    {
      m_data = data;
      return BUF_SUCCESS;
    }

    virtual int get(DataType& value)
    {
      value = m_data;
      return BUF_SUCCESS;
    }

    virtual DataType& getRef()
    {
      return m_data;
    }

  private:
    DataType m_data;
    long int m_length;
  };


}; // namespace RTC
#endif // BufferBase_h
