// -*- C++ -*-
/*!
 * @file CORBA_Util.h
 * @brief CORBA utility template classes
 * @date $Date: 2007/04/26 15:24:41 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: CORBA_SeqEx.h,v 1.3 2007/04/26 15:24:41 n-ando Exp $
 *
 */

/*
 * $Log: CORBA_SeqEx.h,v $
 * Revision 1.3  2007/04/26 15:24:41  n-ando
 * The header include order was modified.
 *
 * Revision 1.2  2006/10/30 08:15:21  n-ando
 * CVS's information tag was revised.
 *
 * Revision 1.1  2006/10/27 09:07:00  n-ando
 * Util.h was moved to CORBA_SeqEx.h.
 *
 * Revision 1.3  2006/10/17 19:07:58  n-ando
 * Helper template function SeqUtil::for_each() for CORBA sequence was added.
 *
 * Revision 1.2  2006/10/17 10:12:18  n-ando
 * SequenceEx's copy constructor and assignment operator were implemented.
 *
 * Revision 1.1  2006/09/11 18:14:53  n-ando
 * The first commit.
 *
 *
 */

#ifndef CORBA_Util_h
#define CORBA_Util_h

#include <rtm/RTC.h>
#include <ace/Guard_T.h>
#include <ace/Thread_Mutex.h>
#include <ace/Recursive_Thread_Mutex.h>


typedef ACE_Guard<ACE_Thread_Mutex> Guard;
typedef ACE_Read_Guard<ACE_Thread_Mutex> Read_Guard;
typedef ACE_Write_Guard<ACE_Thread_Mutex> Write_Guard;

typedef ACE_Guard<ACE_Recursive_Thread_Mutex> Guard_r;
typedef ACE_Read_Guard<ACE_Recursive_Thread_Mutex> Read_Guard_r;
typedef ACE_Write_Guard<ACE_Recursive_Thread_Mutex> Write_Guard_r;


/*!
 * CORBA sequence إѡƥץ졼ȴؿ
 *
 * for_each ()
 * find ()
 * push_back ()
 * insert ()
 * front()
 * back ()
 * erase ()
 * clear ()
 *
 */
namespace CORBA_Sequence_Util
{
  /*!
   * @if jp
   * @brief CORBA sequence Ф functor ŬѤ
   *
   * CORBA sequence ƤǤФơͿ줿 functor ŬѤ롣
   * functor  void functor(CORBA sequence ) ηȤɬפ롣
   *
   * @return ƤǤ Functor
   * @param seq Functor ŬѤ CORBA sequence
   * @param functor CORBA sequence Ǥ Functor
   *
   * @else
   *
   * @brief Apply the functor to all CORBA sequence elements
   *
   * Apply the given functor to the given CORBA sequence.
   * functor should be void functor(CORBA sequence element).
   *
   * @return Functor that processed all CORBA sequence elements
   * @param seq CORBA sequence to be applied the functor
   * @param functor A functor to process CORBA sequence elements
   *
   * @endif
   */
  template <class CorbaSequence, class Functor>
  Functor for_each(const CorbaSequence& seq, Functor f)
  {
    CORBA::ULong len;
    len = seq.length();
    for (CORBA::ULong i = 0; i < len; ++i)
      {
	f(seq[i]);
      }
    return f;
  }


  /*!
   * @if jp
   * @brief CORBA sequence 椫 functor Ŭ礹ǤΥǥå֤
   *
   * CORBA sequence ƤǤФơͿ줿 functor ŬѤ
   * functor  true ֤褦Υǥå֤
   * functor  bool functor(const CORBA sequence ) ηȤꡢ
   * Ŭ礹ǤФ true ֤ɬפ롣
   *
   * @return Functor Ŭ礹ǤΥǥå
   *         ĤʤȤ -1 ֤
   * @param seq Functor ŬѤ CORBA sequence
   * @param functor CORBA sequence Ǥ򸫤Ĥ Functor
   *
   * @else
   *
   * @brief Return the index of CORBA sequence element that functor matches 
   *
   * This operation applies the given functor to the given CORBA sequence,
   * and returns the index of the sequence element that the functor matches.
   * The functor should be bool functor(const CORBA sequence element) type,
   * and it would return true, if the element matched the functor.
   *
   * @return The index of the element that functor matches.
   *         If no element found, it would return -1.
   * @param seq CORBA sequence to be applied the functor
   * @param functor A functor to process CORBA sequence elements
   *
   * @endif
   */
  template <class CorbaSequence, class Functor>
  CORBA::Long find(const CorbaSequence& seq, Functor f)
  {
    CORBA::ULong len(seq.length());
    for (CORBA::ULong i = 0; i < len; ++i)
      {
	if (f(seq[i])) return (CORBA::Long)i;
      }
    return -1;
  }

  /*!
   * @if jp
   * @brief CORBA sequence κǸǤɲä
   *
   * CORBA sequence κǸͿ줿Ǥɲä롣
   * CORBA sequence ĹϼưŪ˳ĥ롣
   *
   * @param seq Ǥɲä CORBA sequence
   * @param elem ɲä
   *
   * @else
   *
   * @brief Push the new element back to the CORBA sequence
   *
   * Add the given element to the last of CORBA sequence.
   * The length of the CORBA sequence will be expanded automatically.
   *
   * @param seq CORBA sequence to be added a new element
   * @param elem The new element to be added to the CORBA sequence
   *
   * @endif
   */
  template <class CorbaSequence, class SequenceElement>
  void push_back(CorbaSequecne& seq, SequenceElement elem)
  {
    CORBA::ULong len(seq.length());
    seq.length(len + 1);
    seq[len] = item;
  }


  /*!
   * @if jp
   * @brief CORBA sequence Ǥ
   *
   * CORBA sequence  index ΰ֤Ǥä롣
   * index  Ϳ줿CORBA sequence κ index 礭
   * ǸǤȤƲä롣
   * CORBA sequence ĹϼưŪ˳ĥ롣
   *
   * @param seq Ǥɲä CORBA sequence
   * @param elem ɲä
   * @param index Ǥɲä
   *
   * @else
   *
   * @brief Insert the element to the CORBA sequence
   *
   * Insert a new element in the given position to the CORBA sequence.
   * If the given index is greater than the length of the sequence,
   * the given element is pushed back to the last of the sequence.
   * The length of the CORBA sequence will be expanded automatically.
   *
   * @param seq The CORBA sequence to be inserted a new element
   * @param elem The new element to be inserted the sequence
   * @param index The inserting position
   *
   * @endif
   */
  template <class CorbaSequence, class SequenceElement>
  void insert(CorbaSequence& seq, SequenceElement& elem, CORBA::ULong index)
  {
    CORBA::ULong len(seq.length());
    if (index > len) push_back(seq, elem);

    seq.length(len + 1);
    for (CORBA::ULong i = len; i >= index; --i)
      {
	seq[i + 1] = seq[i];
      }
    seq[index] = elem;
  }


  /*!
   * @if jp
   * @brief CORBA sequence ƬǤ
   *
   * seq[0] Ʊ
   *
   * @param seq Ǥ CORBA sequence
   *
   * @else
   *
   * @brief Get the front element of the CORBA sequence
   *
   * This operation returns seq[0].
   *
   * @param seq The CORBA sequence to be get the element
   *
   * @endif
   */
  template <class CorbaSequence, class SequenceElement>
  SequenceElement& front(CorbaSequence& seq)
  {
    return seq[0];
  }


  /*!
   * @if jp
   * @brief CORBA sequence Ǥ
   *
   * seq[seq.length() - 1] Ʊ
   *
   * @param seq Ǥ CORBA sequence
   *
   * @else
   *
   * @brief Get the last element of the CORBA sequence
   *
   * This operation returns seq[seq.length() - 1].
   *
   * @param seq The CORBA sequence to be get the element
   *
   * @endif
   */	
  template <class CorbaSequence, class SequenceElement>
  SequenceElement& back(CorbaSequence& seq)
  {
    return seq[seq.length() - 1];
  }


  /*!
   * @if jp
   * @brief CORBA sequence λꤵ줿֤Ǥ
   *
   * ꤵ줿ǥåǤ롣
   * 줿Ǥϵͤ졢sequence Ĺ1롣
   *
   * @param seq Ǥ CORBA sequence
   * @param index ǤΥǥå
   *
   * @else
   *
   * @brief Erase the element of the specified index
   *
   * This operation removes the element of the given index.
   * The other elements are closed up around the hole.
   *
   * @param seq The CORBA sequence to be get the element
   * @param index The index of the element to be removed
   *
   * @endif
   */
  template <class CorbaSequence>
  void erase(CorbaSequence& seq, CORBA::ULong index)
  {
    CORBA::ULong len(seq.length());
    if (index > len) return;

    for (CORBA::ULong i = index; i < len - 1; ++i)
      {
	seq[i] = seq[i + 1];
      }
    seq.length(len - 1);
  }

  /*!
   * @if jp
   * @brief CORBA sequence Ǥ
   *   
   * seq.length(0) Ʊ
   *
   * @else
   *
   * @brief Erase all the elements of the CORBA sequence
   *
   * same as seq.length(0).
   *
   * @endif
   */
  template <class CorbaSequence>
  void clear(CorbaSequence& seq)
  {
    seq.length(0);
  }
  // End of CORBA sequence helper functions



  //============================================================
  // CORBA sequence extention class
  //============================================================

  template <class T>
  struct LockedStruct
  {
    ACE_Thread_Mutex lock;
    T                data;
  };
  
  /*!
   * @if jp
   * @class SequenceEx
   * @brief CORBA::sequence ĥ饹
   *
   * Υ饹 CORBA  sequence ĥ std::vector Υ󥿡ե
   * 󶡤 (㤨 size(), max_size(), empty(), resize(), insert(),
   * erase(), erase_if(), push_back(), pop_back(), find()).
   * CORBA  sequence ѾƤ뤿ᡢCORBA  sequence 
   * ڥ졼(like operator=(), maximum(), length(), operator[])
   * ѲǽǤ롣
   * .
   * @else
   *
   * @brief CORBA::sequence extention class
   *
   * This class extends CORBA sequence type, and provides std::vector like
   * interfaces (like size(), max_size(), empty(), resize(), insert(),
   * erase(), erase_if(), push_back(), pop_back(), find()).
   * Since this class inherits CORBA sequence class, user can also use CORBA
   * sequence interfaces (like operator=(), maximum(), length(), operator[]).
   *
   * @endif
   */ 
  template <class CorbaSequence, class SequenceItem, class Mutex>
  class SequenceEx
    : public CorbaSequence
  {
  public:
    /*!
     * @if jp
     *
     * @brief ǥեȥ󥹥ȥ饯
     *
     * ǥեȥ󥹥ȥ饯ǥեȥ󥹥ȥ饯Ǥϡ
     * 쥯饹CORBA::sequence Ĺ 0 ˥åȤ롣
     * 
     * @param CorbaSequence CORBA IDL  sequence 
     * @param SeuenceItem CorbaSequence Ƿ
     * @param Mutex Mutexη
     *
     * @else
     *
     * @brief Default constructor
     *
     * Default constructor. This default constructor sets the sequence length
     * of CorbaSequence base class.
     *
     * @param CorbaSequence Sequence type defined in CORBA IDL
     * @param SeuenceItem The item type of CorbaSequence
     * @param Mutex The type of mutex
     *
     * @endif
     */
    SequenceEx(): CorbaSequence(0) {};
    
    
    /*!
     * @if jp
     *
     * @brief CorbaSequence Υԡ󥹥ȥ饯
     *
     * CorbaSequenceΥԡ󥹥ȥ饯
     * Ϳ줿 CorbaSequence Ƥ򥳥ԡ롣
     *
     * @param _sq CorbaSequence Υԡ
     *
     * @else
     *
     * @brief Copy constructor from CorbaSequence
     *
     * This constructor copies sequence contents from given CorbaSequence
     * to this object.
     *
     * @param _sq Copy source of CorbaSequence type
     *
     * @endif
     */
    SequenceEx(const CorbaSequence& _sq)
      : CorbaSequence(0)
    {
      this->length(_sq.length());
      CORBA::ULong len(this->length());
      for (CORBA::ULong i = 0; i < len; ++i)
	(*this)[i] = _sq[i];
    };
    
    
    /*!
     * @if jp
     *
     * @brief ԡ󥹥ȥ饯
     *
     * @param _sq ԡ
     *
     * @else
     *
     * @brief Copy constructor
     *
     * @param _sq Copy source.
     *
     * @endif
     */
    SequenceEx(const SequenceEx& _sq)
      : CorbaSequence(0)
    {
      this->length(_sq.length());
      CORBA::ULong len(this->length());
      for (CORBA::ULong i = 0; i < len; ++i)
	(*this)[i] = _sq[i];
    };
    
    
    /*!
     * @if jp
     *
     * @brief 黻
     *
     * @param _sq ԡ
     *
     * @else
     *
     * @brief Assignment operator
     *
     * @param _sq Copy source.
     *
     * @endif
     */
    SequenceEx& operator=(const SequenceEx& _sq)
    {
      this->length(_sq.length());
      CORBA::ULong len(this->length());
      for (CORBA::ULong i = 0; i < len; ++i)
	(*this)[i] = _sq[i];
      return *this;
    };
    
    
    /*!
     * @if jp
     *
     * @brief CorbaSequence 黻
     *
     * @param _sq ԡ
     *
     * @else
     *
     * @brief Assignment operator from CorbaSequence
     *
     * @param _sq Copy source.
     *
     * @endif
     */
    SequenceEx& operator=(const CorbaSequence& _sq)
    {
      this->length(_sq.length());
      CORBA::ULong len(this->length());
      for (CORBA::ULong i = 0; i < len; ++i)
	(*this)[i] = _sq[i];
      return *this;
    };
    
    
    /*!
     * @if jp
     *
     * @brief ǥȥ饯
     *
     * @else
     *
     * @brief Destructor
     *
     * @endif
     */
    virtual ~SequenceEx()
    {
      this->length(0);
    };
    
    
    /*!
     * @if jp
     *
     * @brief 
     *
     * Υڥ졼ϥ󥹤Υ֤
     * CorbaSequence::length() Ʊ
     *
     * @return 󥹤Υ
     *
     * @else
     *
     * @brief Get size of this sequence
     *
     * This operation returns the size of the sequence.
     * This is same as CorbaSequence::length().
     *
     * @return The size of the sequence.
     *
     * @endif
     */
    inline CORBA::ULong size() { return this->length(); }
    
    
    /*!
     * @if jp
     *
     * @brief ǼǽʺΥ
     *
     * Υڥ졼ϥ󥹤θߤγǼǽʺΥ֤
     * CorbaSequence::maximum() Ʊ
     *
     * @return 󥹤˳ǼǽʺΥ
     *
     * @else
     *
     * @brief Get current maximum size of this sequence
     *
     * This operation returns the current maximum size of the sequence.
     * This is same as CorbaSequence::maximum().
     *
     * @return The maximum size of the sequence.
     *
     * @endif
     */
    inline CORBA::ULong max_size() { return this->maximum(); }
    
    
    /*!
     * @if jp
     *
     * @brief 󥹤ɤĴ٤
     *
     * Υڥ졼ϥ󥹤ɤ bool ֤ͤ
     *  0 ʤ trueǤʤ false ֤
     *
     * @return 󥹤ɤ bool 
     *
     * @else
     *
     * @brief Test whether the sequence is empty
     *
     * This operation returns bool value whether the sequence is empty.
     * If the size of the sequence is 0, this operation returns true,
     * and in other case this operation returns false.
     *
     * @return The bool value whether the sequence is empty.
     *
     * @endif
     */
    inline bool empty() { return (this->length() == 0) ? true : false; }
    
    
    /*!
     * @if jp
     *
     * @brief 󥹤ꥵ
     *
     * Υڥ졼ϥ󥹤Ĺѹ롣
     * ߤĹ礭ʥͿ줿硢 x ǡ
     * ˥Ȥ줿ʬ롣
     * ߤĹ꾮Ϳ줿硢CorabSequence Ʊͤ
     * ;ʬʥ󥹤ǤϺ롣
     *
     * @param new_size 󥹤Υ
     * @param itemĹʤäʬΥ󥹤
     *
     * @else
     *
     * @brief Resize the length of the sequence
     *
     * This operation resizes the length of the sequence.
     * If longer length than current sequence length is given,
     * newly allocated rooms will be assigned by element given by the argument.
     * If shorter length than current sequence length is given,
     * the excessive element of a sequence is deleted like behavior of
     * CorabSequence
     *
     * @param new_size The new size of the sequence
     * @param item   Sequence element to be assigned to new rooms.
     *
     * @endif
     */
    void resize(CORBA::ULong new_size, SequenceItem& item)
    {
      ACE_Write_Guard<Mutex> gaurd(lock);
      CORBA::ULong len(this->length);
      if (new_size > len) // Widen sequence
	{
	  this->length(new_size);
	  for (CORBA::ULong i = len; i < new_size; ++i)
	    (*this)[i] = item;
	}
      else if (new_size < len) // shorten sequence
	{
	  this->length(new_size);
	}
    }
    
    
    /*!
     * @if jp
     *
     * @brief 󥹤Ǥ
     *
     * Υڥ졼ϥ󥹤Ǥ롣
     *
     * @param position Ǥ
     * @param item륷󥹤
     *
     * @else
     *
     * @brief Insert a new item to the sequence
     *
     * This operation inserts a new item to the sequence.
     *
     * @param position The position of new inserted item.
     * @param item   Sequence element to be inserted.
     *
     * @endif
     */
    void insert(CORBA::ULong position, const SequenceItem& item)
    {
      ACE_Write_Guard<Mutex> gaurd(lock);
      CORBA::ULong len(this->length());
      // allowed position: [0, len]
      if (position > len) throw; // InvalidParameter("Invalid index");
      
      // widen sequence
      this->length(len + 1);
      // shift items
      for (CORBA::ULong i = (len - 1); i > position; --i)
	{
	  (*this)[i] = (*this)[i-1];
	}
      (*this)[position] = item;
    }
    
    
    /*!
     * @if jp
     *
     * @brief 󥹤Ǥ
     *
     * Υڥ졼ϥ󥹤Ǥ
     *
     * @param position 륷Ǥξ
     *
     * @else
     *
     * @brief Erase an item of the sequence
     *
     * This operation erases an item from the sequence.
     *
     * @param position The position of erased item.
     *
     * @endif
     */
    SequenceItem erase(CORBA::ULong position)
    {
      ACE_Write_Guard<Mutex> gaurd(lock);
      CORBA::ULong len(this->length());
      // allowed position: [0, len)
      if (position > (len - 1)) throw; // InvalidParameter("Invalid index");
      
      // keep erased item
      SequenceItem erased((*this)[position]);
      // shift remaining items
      for (CORBA::ULong i = position; i < (len - 1); ++i)
	{
	  (*this)[i] = (*this)[i+1];
	}
      // shorten sequence
      this->length(len - 1);
      return erased;
    }
    
    /*!
     * @if jp
     *
     * @brief 󥹤ǤҸΤäƺ
     *
     * Υڥ졼ϽҸȤͿ줿ؿ֥Ȥ
     * 郎ΤȤΥ󥹤Ǥ롣
     *
     * @param f 륷󥹤ꤹѸ
     *
     * @else
     *
     * @brief Erase an item according to the given predicate
     *
     * This operation erases an item according to the given predicate.
     *
     * @param f The predicate functor to decide deletion.
     *
     * @endif
     */
    template <class Predicate>
    SequenceItem erase_if(Predicate f)
    {
      ACE_Write_Guard<Mutex> gaurd(lock);
      CORBA::ULong len(this->length());
      for (CORBA::ULong i = 0; i < len; ++i)
	if (f((*this)[i]))
	  return erase(i);
      throw;
    }
    
    
    /*!
     * @if jp
     *
     * @brief ǤǸɲä
     *
     * Υڥ졼Ϳ줿Ǥ򥷡󥹤κǸɲä롣
     *
     * @param item ɲä뤹륪֥
     *
     * @else
     *
     * @brief Append an item to the end of the sequence.
     *
     * This operation push back an item to the of the sequence.
     *
     * @param item The object to be added to the end of the sequnce.
     *
     * @endif
     */ 
    void push_back(const SequenceItem& item)
    {
      ACE_Write_Guard<Mutex> gaurd(lock);
      CORBA::ULong len(this->length());
      this->length(len + 1);
      (*this)[len] = item;
    }
    
    void pop_back()
    {
      ACE_Write_Guard<Mutex> gaurd(lock);
      CORBA::ULong len(this->length());
      this->len(len - 1);
    }
    
    template <class F>
    SequenceItem find(F f) const
    {
      ACE_Read_Guard<Mutex> guard(lock);
      CORBA::ULong len(this->length());
      for (CORBA::ULong i = 0; i < len; ++i)
	if (f((*this)[i]))
	  return (*this)[i];
      throw; // InvalidParameter("Not found.");
    }
    
    mutable Mutex lock;
  }; // class SequenceEx
  
  
}; // namespace CORBA_Sequence_Util

#endif // CORBA_Util_h
