// -*- C++ -*-
/*!
 * @file CorbaConsumer.h
 * @brief CORBA Consumer class
 * @date $Date: 2007/01/21 09:05:13 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: CorbaConsumer.h,v 1.5 2007/01/21 09:05:13 n-ando Exp $
 *
 */

/*
 * $Log: CorbaConsumer.h,v $
 * Revision 1.5  2007/01/21 09:05:13  n-ando
 * Template arguments "_ptr_type" and "_var_type" are defined as typename.
 * The given object references are duplicated.
 *
 * Revision 1.4  2007/01/12 14:29:18  n-ando
 * A trivial bug fix.
 *
 * Revision 1.3  2007/01/06 17:39:33  n-ando
 * The copy constructor and the substitution operator were added.
 *
 * Revision 1.2  2007/01/04 00:48:20  n-ando
 * - Class name has changed.
 * - operator->() was added.
 * - CorbaConsumer::releaseObject() was added.
 *
 * Revision 1.1  2006/11/21 08:32:26  n-ando
 * ConsumerBase class and Cosnumer class were added for proxy placeholder.
 *
 */

#ifndef Consumer_h
#define Consumer_h
#include <iostream>
#ifdef ORB_IS_MICO
#include <CORBA.h>
#endif
#ifdef ORB_IS_OMNIORB
#include <omniORB4/CORBA.h>
#endif
#ifdef ORB_IS_ORBACUS
#include <OB/CORBA.h>
#endif
#ifdef ORB_IS_ORBIT2
#include <orbitcpp/orb-cpp/orbitcpp.h>
#endif
#ifdef ORB_IS_ORBIX
#include <CORBA.h>
#endif
#ifdef ORB_IS_TAO
#include <tao/corba.h>
#endif

namespace RTC
{
  /*!
   * @if jp
   * @class ConsumerBase
   * @brief ֥ȥե󥹤ݻץ졼ۥ쥯饹
   * @else
   * @class ConsumerBase
   * @brief Placeholder base class to hold remote object reference.
   * @endif
   */
  class CorbaConsumerBase
  {
  public:
    /*!
     * @if jp
     * @brief 󥹥ȥ饯
     * @else
     * @brief Consructor
     * @endif
     */
    CorbaConsumerBase(){};

    /*!
     * @if jp
     * @brief ԡ󥹥ȥ饯
     * @else
     * @brief Copy Consructor
     * @endif
     */
    CorbaConsumerBase(const CorbaConsumerBase& x)
    {
      m_objref = x.m_objref;
    }

    CorbaConsumerBase& operator=(const CorbaConsumerBase& x)
    {
      m_objref = x.m_objref;
      return *this;
    }

    /*!
     * @if jp
     * @brief ǥȥ饯
     * @else
     * @brief Destructor
     * @endif
     */
    virtual ~CorbaConsumerBase(){};


    /*!
     * @if jp
     *
     * @brief CORBA֥Ȥ򥻥åȤ
     *
     * Ϳ줿֥ȥե󥹤ϡConsumerBase ֥
     * CORBA::Object_var Ȥݻ롣 
     *
     * @param obj CORBA ֥ȤΥե
     * @return obj  nil ե󥹤ξ false ֤
     *
     * @else
     *
     * @brief Set CORBA Object
     *
     * The given CORBA Object is held as CORBA::Object_var type
     *
     * @param obj Object reference of CORBA object
     * @return If obj is nil reference, it returns false.
     *
     * @endif
     */
    virtual bool setObject(CORBA::Object_ptr obj)
    {
      if (CORBA::is_nil(obj))
	{
	  return false;
	}
      m_objref = CORBA::Object::_duplicate(obj);
      return true;
    }

    /*!
     * @if jp
     *
     * @brief CORBA֥Ȥ
     *
     * Ϳ줿֥ȥե󥹤ϡConsumerBase ֥
     * CORBA::Object_var Ȥݻ롣 
     *
     * @param obj CORBA ֥ȤΥե
     *
     * @else
     *
     * @brief Set CORBA Object
     *
     * The given CORBA Object is held as CORBA::Object_var type
     *
     * @param obj Object reference of CORBA object
     *
     * @endif
     */
    virtual CORBA::Object_ptr getObject()
    {
      return m_objref;
    }

    virtual void releaseObject()
    {
      m_objref = CORBA::Object::_nil();
    }

  protected:
    CORBA::Object_var m_objref;
  };



  /*!
   * @if jp
   *
   * @class Consumer
   * @brief ֥ȥե󥹤ݻץ졼ۥƥץ졼ȥ饹
   * 
   * ƥץ졼ȰͿ줿Υ֥Ȥݻ롣
   * ֥ȤåȤ줿ȤˡͿ줿 narrow Τǡ
   * _ptr() Ǽե󥹤ϡnarrow ѤߤΥե󥹤Ǥ롣
   * ŪʻѤΤˡ_ptr , _varƱ˥ƥץ졼ȰȤ
   * Ϳɬפ롣(ջ໲)
   * <br>  
   * : ObjectTypePtr = ObjectType::_ptr_type ȤƤ뤫
   *       _ptr_type ɸǤϵꤵƤʤ
   *       omniORB, TAO, MICO Ǥϡץ饹ǡ
   *       Type_ptr  _ptr_type  typedef ƤΤǡ
   *       ƥץ졼Ȥ2ꤷʤƤ⥳ѥ̤롣
   *
   * @param ObjectType Υۥݻ륪֥Ȥη
   * @param ObjectTypePtr Υۥݻ _ptr 
   * @param ObjectTypeVar Υۥݻ _var 
   *
   * @else
   *
   * @class Consumer
   * @brief Placeholder template class to hold remote object reference.
   *
   * This class holds a type of object that given by template parameter.
   * For internal use, _ptr type and _var type should be given as template
   * parameter. (Please refere the following notation.)
   *
   * Note: ObjectTypePtr's default value is defined as ObjectType::_ptr_type,
   *       although _ptr_type is not defined as normative type.
   *       However, omniORB, TAO, MICO, ORBit, ORBacus define _ptr_type and
   *       _var_type as _ptr type and _var type in stub code.
   *       Usually, you don't need to specify 2nd and 3rd template arguments.
   *       
   * @endif
   */
  template <class ObjectType,
	    typename ObjectTypePtr = typename ObjectType::_ptr_type,
	    typename ObjectTypeVar = typename ObjectType::_var_type>
  class CorbaConsumer
    : public CorbaConsumerBase
  {
  public:
    /*!
     * @if jp
     * @brief 󥹥ȥ饯
     * @else
     * @brief Consructor
     * @endif
     */
    CorbaConsumer(){};

    CorbaConsumer(const CorbaConsumer& x)
    {
      m_var = x.m_var;
    }

    CorbaConsumer& operator=(const CorbaConsumer& x)
    {
      m_var = x.m_var;
    }

    /*!
     * @if jp
     * @brief ǥȥ饯
     * @else
     * @brief Destructor
     * @endif
     */
    virtual ~CorbaConsumer()
    {
    };

    /*!
     * @if jp
     * @brief ֥Ȥ򥻥åȤ
     *
     * ConsumerBase ΥС饤ɡCORBA::Object_var ˥֥Ȥ򥻥å
     * ȤȤˡtemplateѥ᡼η narrow ֥Ȥ
     * ѿݻ롣
     *
     * @param obj CORBA Objecct
     * 
     * @else
     * @brief Set Object
     *
     * Override function of ConsumerBase. This operation set an Object to 
     * CORBA:Object_var in the class, and this object is narrowed to
     * given template parameter and stored in the member variable.
     *
     * @endif
     */
    virtual bool setObject(CORBA::Object_ptr obj)
    {
      if (CorbaConsumerBase::setObject(obj))
	{
	  m_var = ObjectType::_duplicate(ObjectType::_narrow(m_objref));
	  if (!CORBA::is_nil(m_var))
	    return true;
	}
      return false; // object is nil
    }


    /*!
     * @if jp
     * @brief ObjectType Υ֥ȤΥե󥹤
     *
     * ObjectType  narrowѤߤΥ֥ȤΥե󥹤롣
     * ֥ȥե󥹤ѤˤϡsetObject() ǥåȺѤߤ
     * ʤФʤʤ
     * ֥ȤåȤƤʤСnil ֥ȥե󥹤
     * ֤롣
     *
     * @return ObjectType  narrow ѤߤΥ֥ȤΥե
     * 
     * @else
     * @brief Get Object reference narrowed as ObjectType
     *
     * This operation returns object reference narrowed as ObjectType.
     * To use the returned object reference, reference have to be set by
     * setObject().
     * If object is not set, this operation returns nil object reference.
     *
     * @return The object reference narrowed as ObjectType
     *
     * @endif
     */
    inline ObjectTypePtr _ptr()
    {
      return m_var;
    }

    /*!
     * @if jp
     * @brief ObjectType Υ֥ȤΥե󥹤
     *
     * ObjectType  narrowѤߤΥ֥ȤΥե󥹤롣
     * ֥ȥե󥹤ѤˤϡsetObject() ǥåȺѤߤ
     * ʤФʤʤ
     * ֥ȤåȤƤʤСnil ֥ȥե󥹤
     * ֤롣
     *
     * @return ObjectType  narrow ѤߤΥ֥ȤΥե
     * 
     * @else
     * @brief Get Object reference narrowed as ObjectType
     *
     * This operation returns object reference narrowed as ObjectType.
     * To use the returned object reference, reference have to be set by
     * setObject().
     * If object is not set, this operation returns nil object reference.
     *
     * @return The object reference narrowed as ObjectType
     *
     * @endif
     */
    inline ObjectTypePtr operator->()
    {
      return m_var;
    }

    virtual void releaseObject()
    {
      CorbaConsumerBase::releaseObject();
      m_var = ObjectType::_nil();
    }

  protected:
    ObjectTypeVar m_var;

  };
}; // namespace RTC
#endif // Consumer_h
