// -*- C++ -*-
/*!
 * @file CorbaNaming.h
 * @brief CORBA naming service helper class
 * @date $Date: 2007/04/13 15:35:21 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: CorbaNaming.h,v 1.2 2007/04/13 15:35:21 n-ando Exp $
 *
 */

/*
 * $Log: CorbaNaming.h,v $
 * Revision 1.2  2007/04/13 15:35:21  n-ando
 * Error handing processing in case NameServer does not exist was added.
 * Some bug fixes.
 *
 * Revision 1.1  2006/11/04 19:43:01  n-ando
 * CORBA Naming service helper class has rewritten and renamed.
 *
 */


#ifndef CorbaNaming_h
#define CorbaNaming_h

#include <rtm/RTC.h>
#ifdef ORB_IS_OMNIORB
#undef PACKAGE_BUGREPORT
#undef PACKAGE_NAME
#undef PACKAGE_STRING
#undef PACKAGE_TARNAME
#undef PACKAGE_VERSION
#include <omniORB4/CORBA.h>
#endif

// STL includes
#include <map>
#include <string>
#include <vector>

namespace RTC
{
  /*!
   * @if jp
   * @class CorbaNaming
   * @brief CORBA Naming Service إѡ饹
   *
   * Υ饹ϡCosNaming::NamingContext Фåѡ饹Ǥ롣
   * CosNaming::NamingContext ĥڥ졼ȤۤƱǽ
   * ڥ졼󶡤ȤȤˡ͡ॳݡͥ CosNaming::Name
   * ʸˤ̾ɽդ륪ڥ졼󶡤롣
   *
   * ֥Ȥ뤤ľ CORBA ͡ॵФ³
   * ʸ塢Υ͡ॵФΥ롼ȥƥȤФƼΥڥ졼
   * 롣
   * ؤΥ͡ߥ󥰥ƥȤκ䥪֥ȤΥХɤˤơ
   * ΥƥȤ¸ߤʤǤ⡢Ū˥ƥȤХ
   * ŪΥƥȤ䥪֥ȤΥХɤԤȤǤ롣
   *
   * @else
   * @class CorbaNaming
   * @brief CORBA Naming Service helper class
   *
   * This class is a wrapper class of CosNaming::NamingContext.
   * Almost the same operations which CosNaming::NamingContext has are
   * provided, and some operation allows string naming representation of
   * context and object instead of CosNaming::Name.
   *
   * The object of the class would connect to a CORBA naming server at
   * the instantiation or immediately after instantiation.
   * After that the object invokes operations to the root context of it.
   * This class realizes forced binding to deep NamingContext, without binding
   * intermediate NamingContexts explicitly.
   *
   * @endif
   */
  class CorbaNaming
  {
  public:
    CorbaNaming(CORBA::ORB_ptr orb);
    CorbaNaming(CORBA::ORB_ptr orb, const char* name_server);
    virtual ~CorbaNaming(){};
    void init(const char* name_server);

    typedef CosNaming::NamingContext::NotFound      NotFound;
    typedef CosNaming::NamingContext::CannotProceed CannotProceed;
    typedef CosNaming::NamingContext::InvalidName   InvalidName;
    typedef CosNaming::NamingContext::AlreadyBound  AlreadyBound;
    typedef CosNaming::NamingContext::NotEmpty      NotEmpty;
    typedef CosNaming::NamingContextExt::InvalidAddress InvalidAddress;
    typedef std::vector<CORBA::Object_ptr> ObjectList;


    /*!
     * @if jp
     *
     * @brief Object  bind 
     *
     * CosNaming::bind() ȤۤƱƯ򤹤뤬Ϳ줿͡ॵФ
     * 롼ȥƥȤФbind()ƤӽФۤʤ롣
     *
     * Name <name>  Object <obj>  NamingContext ˥Хɤ롣
     * c_n  n ܤ NameComponent 򤢤魯Ȥȡ
     * name  n Ĥ NameComponent ȤʲΤ褦˰롣
     *
     * cxt->bind(<c_1, c_2, ... c_n>, obj) ϰʲƱǤ롣
     * cxt->resolve(<c_1, ... c_(n-1)>)->bind(<c_n>, obj)
     *
     * ʤ1ܤn-1ܤΥƥȤ褷n-1ܤΥƥ
     *  name <n> Ȥơobj  bind 롣
     * ̾˻ä <c_1, ... c_(n-1)>  NemingContext ϡ
     * bindContext()  rebindContext() Ǵ˥ХɺѤߤǤʤФʤʤ
     * ⤷ <c_1, ... c_(n-1)>  NamingContext ¸ߤʤˤϡ
     * NotFound 㳰ȯ롣
     *
     * Хɥե饰 force  true λϡ<c_1, ... c_(n-1)>
     * ¸ߤʤˤ⡢ƵŪ˥ƥȤХɤʤ顢
     * ǽŪ obj ̾ name <c_n> ˥Хɤ롣
     *
     * ξǤ⡢n-1ܤΥƥȾ name<n> Υ֥
     * (Object 뤤 ƥ) ХɤƤ
     * AlreadyBound 㳰ȯ롣
     *
     * @param name ֥Ȥդ̾ NameComponent
     * @param obj Ϣդ Object
     * @param force trueξ硢ΥƥȤŪ˥Хɤ
     *
     * @exception NotFound  <c_1, c_2, ..., c_(n-1)> ¸ߤʤ
     * @exception CannotProceed 餫ͳǽ³Ǥʤ
     * @exception InvalidName  name ̾
     * @exception AlreadyBound name <c_n>  Object Ǥ˥ХɤƤ롣
     *
     * @else
     *
     * @brief
     *
     * @endif
     */
    void bind(const CosNaming::Name& name, CORBA::Object_ptr obj,
	      const bool force = 1)
      throw(NotFound, CannotProceed, InvalidName, AlreadyBound);


    /*!
     * @if jp
     *
     * @brief Object  bind 
     *
     * Object  bind ݤͿ̾ʸɽǤ뤳Ȱʳϡbind()
     * ƱǤ롣bind(toName(string_name), obj) 
     *
     * @param string_name ֥Ȥդ̾ʸɽ
     * @param obj Ϣդ륪֥
     * @param force trueξ硢ΥƥȤŪ˥Хɤ
     *
     * @exception NotFound  <c_1, c_2, ..., c_(n-1)> ¸ߤʤ
     * @exception CannotProceed 餫ͳǽ³Ǥʤ
     * @exception InvalidName  name ̾
     * @exception AlreadyBound name <n>  Object Ǥ˥ХɤƤ롣
     *
     * @else
     *
     * @brief
     *
     * @endif
     */
    void bindByString(const char* string_name, CORBA::Object_ptr obj,
		      const bool force = 1)
      throw(NotFound, CannotProceed, InvalidName, AlreadyBound);

    /*!
     * @if jp
     *
     * @brief ΥƥȤ bind ʤ Object  bind 
     *
     * context Ϳ줿 NamingContext Фơname ǻꤵ줿
     * ͡ॳݡͥ <c_1, ... c_(n-1)>  NamingContext Ȥ
     * 褷ʤ顢̾ <c_n> Ф obj  bind 롣
     * ⤷<c_1, ... c_(n-1)> б NamingContext ʤˤ
     *  NamingContext Хɤ롣
     *
     * ǽŪ <c_1, c_2, ..., c_(n-1)> б NamingContext 
     * ޤϲ褵줿ǡCosNaming::bind(<c_n>, object) ƤӽФ롣
     * ΤȤǤ˥Хǥ󥰤¸ߤ AlreadyBound㳰ȯ롣
     *
     * ΥƥȤ褹ǡ褷褦Ȥ륳ƥȤ
     * Ʊ̾ NamingContext ǤϤʤ Binding ¸ߤ硢
     * CannotProceed 㳰ȯߤ롣
     *
     * @param context bind 򳫻Ϥ롡NamingContext
     * @param name ֥Ȥդ̾Υ͡ॳݡͥ
     * @param obj Ϣդ륪֥
     *
     * @exception CannotProceed <c_1, ..., c_(n-1)> б NamingContext 
     *            ΤҤȤĤǤ NamingContext ʳ object ˥Х
     *            Ƥꡢ³Ǥʤ
     * @exception InvalidName ̾ name 
     * @exception AlreadyBound name <c_n> ˤǤ˲餫 object Х
     *            Ƥ롣
     * @else
     *
     * @brief
     *
     * @endif
     */
    void bindRecursive(CosNaming::NamingContext_ptr context,
		       const CosNaming::Name& name,
		       CORBA::Object_ptr obj)
      throw(CannotProceed, InvalidName, AlreadyBound);



    /*!
     * @if jp
     *
     * @brief Object  rebind 
     *
     * name ǻꤵ줿 Binding Ǥ¸ߤ bind() Ʊ
     * Ǥ롣Хǥ󥰤Ǥ¸ߤˤϡХǥ󥰤
     * ֤롣
     *
     * @param name ֥Ȥդ̾ NameComponent
     * @param obj Ϣդ륪֥
     * @param force trueξ硢ΥƥȤŪ˥Хɤ
     *
     * @else
     *
     * @brief
     *
     * @endif
     */
    void rebind(const CosNaming::Name& name, CORBA::Object_ptr obj,
		const bool force = 1)
      throw(NotFound, CannotProceed, InvalidName);


    /*!
     * @if jp
     *
     * @brief Object  rebind 
     *
     * Object  rebind ݤͿ̾ʸɽǤ뤳Ȱʳ rebind()
     * ƱǤ롣rebind(toName(string_name), obj) 
     *
     * @param string_name ֥Ȥդ̾ʸɽ
     * @param obj Ϣդ륪֥
     * @param force trueξ硢ΥƥȤŪ˥Хɤ
     *
     * @exception NotFound  <c_1, c_2, ..., c_(n-1)> ¸ߤʤ
     * @exception CannotProceed 餫ͳǽ³Ǥʤ
     * @exception InvalidName  name ̾
     *
     * @else
     *
     * @brief
     *
     * @endif
     */
    void rebindByString(const char* string_name, CORBA::Object_ptr obj,
			const bool force = 1)
      throw(NotFound, CannotProceed, InvalidName);

    
    /*!
     * @if jp
     *
     * @brief ΥƥȤ bind ʤ Object  rebind 
     *
     * name <c_n> ǻꤵ줿 NamingContext ⤷ Object Ǥ¸ߤ
     *  bindRecursive() ƱǤ롣
     *
     * name <c_n> ǻꤵ줿Хǥ󥰤Ǥ¸ߤˤϡ
     * Хǥ󥰤֤롣
     *
     * @param name ֥Ȥդ̾ʸɽ
     * @param obj Ϣդ륪֥
     * @param force trueξ硢ΥƥȤŪ˥Хɤ
     *
     * @exception CannotProceed ΥƥȤǤʤ
     * @exception InvalidName Ϳ줿 name 
     *
     * @else
     *
     * @brief
     *
     * @endif
     */
    void rebindRecursive(CosNaming::NamingContext_ptr context,
			 const CosNaming::Name& name,
			 CORBA::Object_ptr obj)
      throw(CannotProceed, InvalidName);

    /*!
     * @if jp
     *
     * @brief NamingContext  bind 
     *
     * bind 륪֥Ȥ NamingContext Ǥ뤳Ȥ bind() 
     * ƱǤ롣
     *
     * @param name ֥Ȥդ̾Υ͡ॳݡͥ
     * @param name_cxt Ϣդ NamingContext
     * @param force trueξ硢ΥƥȤŪ˥Хɤ
     *
     * @else
     *
     * @brief
     *
     * @endif
     */
    void bindContext(const CosNaming::Name& name,
		     CosNaming::NamingContext_ptr name_cxt,
		     const bool force = 1)
      throw(NotFound, CannotProceed, InvalidName, AlreadyBound);

    /*!
     * @if jp
     *
     * @brief NamingContext  bind 
     *
     * bind 륪֥Ȥ NamingContext Ǥ뤳Ȥ bind() 
     * ƱǤ롣
     *
     * @param name ֥Ȥդ̾ʸɽ
     * @param name_cxt Ϣդ NamingContext
     * @param force trueξ硢ΥƥȤŪ˥Хɤ
     *
     * @else
     *
     * @brief
     *
     * @endif
     */
    void bindContext(const char* string_name,
		     CosNaming::NamingContext_ptr name_cxt,
		     const bool force = 1)
      throw(NotFound, CannotProceed, InvalidName, AlreadyBound);

    /*!
     * @if jp
     *
     * @brief NamingContext  bind 
     *
     * bind 륪֥Ȥ NamingContext Ǥ뤳Ȥ
     * bindRecursive() ƱǤ롣
     *
     * @param context bind 򳫻Ϥ롡NamingContext
     * @param name ֥Ȥդ̾Υ͡ॳݡͥ
     * @param name_cxt Ϣդ NamingContext
     * @param force trueξ硢ΥƥȤŪ˥Хɤ
     *
     * @else
     *
     * @brief
     *
     * @endif
     */
    void bindContextRecursive(CosNaming::NamingContext_ptr context,
			      const CosNaming::Name& name,
			      CosNaming::NamingContext_ptr name_cxt);
    /*!
     * @if jp
     *
     * @brief NamingContext  rebind 
     *
     * name ǻꤵ줿ƥȤǤ¸ߤ bindContext() 
     * ƱǤ롣
     * Хǥ󥰤Ǥ¸ߤˤϡХǥ󥰤
     * ֤롣
     *
     * @param name ֥Ȥդ̾Υ͡ॳݡͥ
     * @param name_cxt Ϣդ NamingContext
     * @param force trueξ硢ΥƥȤŪ˥Хɤ
     *
     * @else
     *
     * @brief
     *
     * @endif
     */
    void rebindContext(const CosNaming::Name& name,
		       CosNaming::NamingContext_ptr name_cxt,
		       const bool force = 1)
      throw(NotFound, CannotProceed, InvalidName);

    /*!
     * @if jp
     *
     * @brief NamingContext  rebind 
     *
     * name ǻꤵ줿ƥȤǤ¸ߤ bindContext() 
     * ƱǤ롣
     * Хǥ󥰤Ǥ¸ߤˤϡХǥ󥰤
     * ֤롣
     *
     * @param name ֥Ȥդ̾ʸɽ
     * @param name_cxt Ϣդ NamingContext
     * @param force trueξ硢ΥƥȤŪ˥Хɤ
     *
     * @else
     *
     * @brief
     *
     * @endif
     */
    void rebindContext(const char* string_name,
		       CosNaming::NamingContext_ptr name_cxt,
		       const bool force = 1)
      throw(NotFound, CannotProceed, InvalidName);


    void rebindContextRecursive(CosNaming::NamingContext_ptr context,
				const CosNaming::Name& name,
				CosNaming::NamingContext_ptr name_cxt);

    /*!
     * @if jp
     *
     * @brief Object  name 褹
     *
     * name  bind Ƥ륪֥ȻȤ֤
     * ͡ॳݡͥ <c_1, c_2, ... c_n> ϺƵŪ˲褵롣
     * 
     * CosNaming::resolve() ȤۤƱƯ򤹤뤬Ϳ줿
     * ͡ॵФΥ롼ȥƥȤФ resolve() ƤӽФ
     * ۤʤ롣
     *
     * @param name 褹٤֥Ȥ̾Υ͡ॳݡͥ
     * @return 褵줿֥Ȼ
     *
     * @else
     *
     * @endif
     */
    CORBA::Object_ptr resolve(const CosNaming::Name& name)
      throw(NotFound, CannotProceed, InvalidName);

    /*!
     * @if jp
     *
     * @brief Object  name 褹
     *
     * name  bind Ƥ륪֥ȻȤ֤
     * ͡ॳݡͥ <c_1, c_2, ... c_n> ϺƵŪ˲褵롣
     * 
     * CosNaming::resolve() ȤۤƱƯ򤹤뤬Ϳ줿
     * ͡ॵФΥ롼ȥƥȤФ resolve() ƤӽФ
     * ۤʤ롣
     *
     * @param name 褹٤֥Ȥ̾ʸɽ
     * @return 褵줿֥Ȼ
     *
     * @else
     *
     * @endif
     */
    CORBA::Object_ptr resolve(const char* string_name)
      throw(NotFound, CannotProceed, InvalidName);

    /*!
     * @if jp
     *
     * @brief ꤵ줿̾Υ֥Ȥ bind 
     *
     * name  bind Ƥ륪֥ȻȤ֤
     * ͡ॳݡͥ <c_1, c_2, ... c_n> ϺƵŪ˲褵롣
     * 
     * CosNaming::unbind() ȤۤƱƯ򤹤뤬Ϳ줿
     * ͡ॵФΥ롼ȥƥȤФ unbind() ƤӽФ
     * ۤʤ롣
     *
     * @param name 褹٤֥Ȥ̾Υ͡ॳݡͥ
     * @return 褵줿֥Ȼ
     *
     * @else
     *
     * @endif
     */
    void unbind(const CosNaming::Name& name)
      throw(NotFound, CannotProceed, InvalidName);

    /*!
     * @if jp
     *
     * @brief ꤵ줿̾Υ֥Ȥ bind 
     *
     * name  bind Ƥ륪֥ȻȤ֤
     * ͡ॳݡͥ <c_1, c_2, ... c_n> ϺƵŪ˲褵롣
     * 
     * CosNaming::unbind() ȤۤƱƯ򤹤뤬Ϳ줿
     * ͡ॵФΥ롼ȥƥȤФ unbind() ƤӽФ
     * ۤʤ롣
     *
     * @param name 褹٤֥Ȥ̾ʸɽ
     * @return 褵줿֥Ȼ
     *
     * @else
     *
     * @endif
     */
    void unbind(const char* string_name)
      throw(NotFound, CannotProceed, InvalidName);

    /*!
     * @if jp
     *
     * @brief ƥȤ
     *
     * Ϳ줿͡ॵо줿 NamingContext ֤
     * ֤줿 NamingContext  bind Ƥʤ
     * 
     * @return 줿 NamingContext
     *
     * @else
     *
     * @endif
     */
    CosNaming::NamingContext_ptr newContext();

    /*!
     * @if jp
     *
     * @brief ƥȤ bind 
     *
     * Ϳ줿 name ФƿƥȤХɤ롣
     * 줿NamingContext ϥ͡ॵо줿ΤǤ롣
     * 
     * @param name NamingContextդ̾Υ͡ॳݡͥ
     * @return 줿 NamingContext
     *
     * @else
     *
     * @endif
     */
    CosNaming::NamingContext_ptr
    bindNewContext(const CosNaming::Name& name, bool force = true)
      throw(NotFound, CannotProceed, InvalidName, AlreadyBound);

    /*!
     * @if jp
     *
     * @brief ƥȤ bind 
     *
     * Ϳ줿 name ФƿƥȤХɤ롣
     * 줿NamingContext ϥ͡ॵо줿ΤǤ롣
     * 
     * @param name NamingContextդ̾ʸɽ
     * @return 줿 NamingContext
     *
     * @else
     *
     * @endif
     */
    CosNaming::NamingContext_ptr
    bindNewContext(const char* string_name, bool force = true)
      throw(NotFound, CannotProceed, InvalidName, AlreadyBound);

    /*!
     * @if jp
     *
     * @brief NamingContext 󥢥ƥֲ
     *
     * context ǻꤵ줿 NamingContext 󥢥ƥֲ롣
     * context ¾ΥƥȤХɤƤ NotEmpty 㳰
     * ȯ롣
     * 
     * @param context 󥢥ƥֲ NamingContext
     *
     * @else
     *
     * @brief Destroy the naming context
     *
     * Delete the specified naming context.
     * any bindings should be <unbind> in which the given context is bound to
     * some names before invoking <destroy> operation on it. 
     *
     * @param context NamingContext which is destroied.
     *     
     * @endif
     */
    void destroy(CosNaming::NamingContext_ptr context)
      throw(NotEmpty);


    /*!
     * @if jp
     * @brief NamingContext ƵŪ˲ä󥢥ƥֲ
     * @else
     * @brief Destroy the naming context recursively
     * @endif
     */
    void destroyRecursive(CosNaming::NamingContext_ptr context)
      throw(NotEmpty, NotFound, CannotProceed, InvalidName);


    /*!
     * @if jp
     * @brief ٤Ƥ Binding 
     * @else
     * @brief Destroy all binding
     * @endif
     */
    void clearAll();


    /*!
     * @if jp
     * @brief Ϳ줿 NamingContext  Binding 
     * @else
     * @brief Get Binding on the NamingContextDestroy all binding
     * @endif
     */
    void list(CosNaming::NamingContext_ptr name_cxt,
	      unsigned long how_many,
	      CosNaming::BindingList_var& bl,
	      CosNaming::BindingIterator_var& bi);


    //============================================================
    // interface of NamingContextExt
    //============================================================
    /*!
     * @if jp
     * @brief Ϳ줿 NameComponent ʸɽ֤
     * @else
     * @brief Get string representation of given NameComponent
     * @endif
     */
    char* toString(const CosNaming::Name& name)
      throw(InvalidName);
    

    /*!
     * @if jp
     * @brief Ϳ줿ʸɽ NameComponent ʬ򤹤
     * @else
     * @brief Get NameComponent from gien string name representation
     * @endif
     */
    CosNaming::Name toName(const char* string_name)
      throw(InvalidName);


    /*!
     * @if jp
     * @brief Ϳ줿 addre  string_name  URLɽ
     * @else
     * @brief Get URL representation from given addr and string_name
     * @endif
     */
    char* toUrl(char* addr, char* string_name)
      throw(InvalidAddress, InvalidName);


    /*!
     * @if jp
     * @brief Ϳ줿ʸɽ resolve ֥Ȥ֤
     * @else
     * @brief Resolve from name of string representation and get object 
     * @endif
     */
    CORBA::Object_ptr resolveStr(const char* string_name)
      throw(NotFound, CannotProceed, InvalidName, AlreadyBound);


    //============================================================
    // Find functions
    //============================================================

    //    ObjectList find(const char* name, const char* kind);
    //    ObjectList findById(const char* name, const char* kind);
    //    ObjectList findByKind(const char* name, const char* kind);

    /*!
     * @if jp
     * @brief ̾Хɤޤϲ褹
     * @else
     * @brief Bind of resolve the given name component
     * @endif
     */
    CORBA::Object_ptr bindOrResolve(CosNaming::NamingContext_ptr context,
				    const CosNaming::Name& name,
				    CORBA::Object_ptr obj);
    
    /*!
     * @if jp
     * @brief ̾Хɤޤϲ褹
     * @else
     * @brief Bind of resolve the given name component
     * @endif
     */
    CosNaming::NamingContext_ptr
    bindOrResolveContext(CosNaming::NamingContext_ptr context,
			 const CosNaming::Name& name,
			 CosNaming::NamingContext_ptr new_context);

    
    /*!
     * @if jp
     * @brief ̾Хɤޤϲ褹
     * @else
     * @brief Bind of resolve the given name component
     * @endif
     */
    CosNaming::NamingContext_ptr
    bindOrResolveContext(CosNaming::NamingContext_ptr context,
			 const CosNaming::Name& name);


    /*!
     * @if jp
     * @brief ͡ॵФ̾
     * @else
     * @brief Get the name of naming server
     * @endif
     */
    const char* getNameServer();
    
    /*!
     * @if jp
     * @brief 롼ȥƥȤ
     * @else
     * @brief Get the root context
     * @endif
     */
    CosNaming::NamingContext_ptr getRootContext();

    
    /*!
     * @if jp
     * @brief ֥Ȥ͡ߥ󥰥ƥȤȽ̤
     * @else
     * @brief Whether the object is NamingContext
     * @endif
     */
    bool isNamingContext(CORBA::Object_ptr obj);
    
    /*!
     * @if jp
     * @brief Ϳ줿̾͡ߥ󥰥ƥȤɤ
     * @else
     * @brief Whether the given name component is NamingContext
     * @endif
     */
    bool isNamingContext(const CosNaming::Name& name);
    
    /*!
     * @if jp
     * @brief Ϳ줿̾͡ߥ󥰥ƥȤɤ
     * @else
     * @brief Whether the given string name is NamingContext
     * @endif
     */
    bool isNamingContext(const char* string_name);
    
    /*!
     * @if jp
     * @brief ͡ॳݡͥȤʬ֤
     * @else
     * @brief Get subset of given name component
     * @endif
     */
    CosNaming::Name subName(const CosNaming::Name& name,
			    long begin,
			    long end = -1);


  protected:
    /*!
     * @if jp
     * @brief ͡ॳݡͥȤʸɽ
     * @else
     * @brief Get string representation of name component
     * @endif
     */
    void nameToString(const CosNaming::Name& name, char* string_name,
		      unsigned long slen);
    /*!
     * @if jp
     * @brief ͡ॳݡͥȤʸɽʸĹ
     * @else
     * @brief Get string length of the name component's string representation
     * @endif
     */
    CORBA::ULong getNameLength(const CosNaming::Name& name);

    /*!
     * @if jp
     * @brief ʸʬ
     * @else
     * @brief Split of string
     * @endif
     */
    unsigned int split(const std::string& input,
		       const std::string& delimiter,
		       std::vector<std::string>& results);


    
    CORBA::ORB_var m_varORB;
    std::string m_nameServer;
    CosNaming::NamingContextExt_var m_rootContext;

  private:
    CORBA::ULong m_blLength;
    
  }; // class CorbaNaming

}; // namespace RTC

#endif // end of __Naming_h__
