// -*- C++ -*-
/*!
 * @file InPort.h
 * @brief InPort template class
 * @date $Date: 2007/09/21 09:15:16 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2003-2005
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: InPort.h,v 1.6.4.3 2007/09/21 09:15:16 n-ando Exp $
 *
 */

/*
 * $Log: InPort.h,v $
 * Revision 1.6.4.3  2007/09/21 09:15:16  n-ando
 * Now InPort's default buffer is RingBuffer.
 *
 * Revision 1.6.4.2  2007/08/20 05:15:39  n-ando
 * Now default buffer is RingBuffer instead of NullBuffer.
 *
 * Revision 1.6.4.1  2007/07/20 15:54:50  n-ando
 * Now ACE_OS::gettimeofday() is used for win32 porting.
 *
 * Revision 1.6  2007/01/06 17:47:51  n-ando
 * Some changes.
 * - Callback declarations are changed.
 * - Return value of write().
 *
 * Revision 1.5  2006/12/02 18:37:29  n-ando
 * A trivial fix.
 *
 * Revision 1.4  2006/11/07 09:26:12  n-ando
 * Renamed RtcInPort.h to InPort.h.
 *
 * Revision 1.3  2005/05/27 07:29:32  n-ando
 * - InPort/OutPort interface was changed.
 *   Exception RTM::InPort::Disconnected -> RTM:PortBase::Disconnected
 *
 * Revision 1.2  2005/05/16 06:12:15  n-ando
 * - Time variables of "m_Value" were initialized in constructors.
 *
 * Revision 1.1.1.1  2005/05/12 09:06:18  n-ando
 * Public release.
 *
 *
 */

#ifndef RtcInPort_h
#define RtcInPort_h

#include <string>
#include <vector>
#include <iostream>
#include <rtm/BufferBase.h>
#include <rtm/RingBuffer.h>
#include <rtm/PortCallBack.h>
#include <rtm/RTC.h>

#define TIMEOUT_TICK_USEC 10
#define USEC_PER_SEC 1000000

namespace RTC
{
  /*!
   * @if jp
   *
   * @class InPort
   *
   * @brief InPort ƥץ졼ȥ饹
   * 
   * InPort μǤ InPortAny<T> Υƥץ졼ȥ饹
   * <T> RTCDataType.idl ˤƤ뷿ǡФȤ
   * Time  tm ,  T data Ĺ¤ΤǤʤƤϤʤʤ
   * InPort ˥󥰥Хåե줿ǡ缡
   * Υ󥰥Хåե˳Ǽ롣󥰥ХåեΥϥǥեȤ64
   * ʤäƤ뤬󥹥ȥ饯ˤꥵꤹ뤳ȤǤ롣
   * ǡϥե饰ˤä̤ɡɾ֤졢isNew(), getNewDataLen()
   * getNewList(), getNewListReverse() Υ᥽åɤˤϥɥ󥰤뤳Ȥ
   * Ǥ롣
   *
   * @else
   *
   * @class InPortAny
   *
   * @brief OutPort template class
   *
   * This class template provides interfaces to input port.
   * Component developer can define input value, which act as input
   * port from other components, using this template.
   * This is class template. This class have to be incarnated class as port
   * value types. This value types are previously define RtComponent IDL.
   * ex. type T: TimedFload, TimedLong etc... 
   *
   * @endif
   */
  template <class DataType,
	    template <class DataType> class Buffer = RingBuffer >
  class InPort
    : public Buffer<DataType>
  {
  public:
    /*!
     * @if jp
     *
     * @brief InPortAny 饹󥹥ȥ饯
     *
     * InPortAny<T> 饹Υ󥹥ȥ饯
     * ѥ᡼ȤͿ T ѿ˥Хɤ롣
     *
     * @param name InPort ̾InPortBase:name() ˤ껲Ȥ롣
     * @param value  InPort ˥Хɤ T ѿ
     * @param bufsize InPort Υ󥰥ХåեΥХåեĹ
     *
     * @else
     *
     * @brief A constructor.
     *
     * Setting channel name and registering channel value.
     *
     * @param name A name of the InPort. This name is referred by
     *             InPortBase::name().
     * @param value A channel value related with the channel.
     * @param bufsize Buffer length of internal ring buffer of InPort 
     *
     * @endif
     */
    InPort(const char* name, DataType& value,
	   int bufsize=64, 
	   bool read_block = false, bool write_block = false,
	   int read_timeout = 0, int write_timeout = 0)
      : Buffer<DataType>(bufsize),
	m_name(name), m_value(value),
	m_readBlock(read_block),   m_readTimeout(read_timeout),
	m_writeBlock(write_block), m_writeTimeout(write_timeout),
	m_OnWrite(NULL), m_OnWriteConvert(NULL),
	m_OnRead(NULL),  m_OnReadConvert(NULL),
	m_OnOverflow(NULL), m_OnUnderflow(NULL)
    {
    };
    
    
    /*!
     * @if jp
     *
     * @brief InPortAny 饹ǥȥ饯
     *
     * InPortAny<T> 饹Υǥȥ饯
     *
     * @else
     *
     * @brief A destructor
     *
     * @endif
     */
    virtual ~InPort(){};



    virtual const char* name()
    {
      return m_name.c_str();
    }


    /*!
     * @if jp
     *
     * @brief DataPort ͤ񤭹
     *
     * DataPort ͤ񤭹ࡣ
     *
     * - Хåե󥯥 OnWrite åȤƤ硢
     *   InPort ݻХåե˽񤭹 OnWrite ƤФ롣
     * - InPort ݻХåեСե򸡽ФǤХåեǤꡢ
     *   ġ񤭹ݤ˥ХåեСե򸡽Ф硢
     *   Хåե󥯥 OnOverflow ƤФ롣
     * - Хåե󥯥 OnWriteConvert åȤƤ硢
     *   Хåե񤭹߻ˡOnWriteConvert  operator()() ͤ
     *   Хåե˽񤭹ޤ롣
     * - setWriteTimeout() ˤ񤭹߻ΥॢȤꤵƤ硢
     *   ॢȻ֤Хåեե֤ΤԤ
     *   OnOverflowåȤƤФƤӽФ롣
     *
     * @else
     *
     * @brief 
     *
     * @endif
     */
    int write(const DataType& value)
    {
      int status = BUF_SUCCESS;

      // std::cout << "InPort::write(const DataType& value)" << std::endl;

      if (m_OnWrite != NULL) (*m_OnWrite)(value);      

      long int timeout = m_writeTimeout;

      timeval tm_cur, tm_pre;
      ACE_Time_Value tt;
      tt = ACE_OS::gettimeofday();
      tm_pre = tt.operator timeval();

      // blocking and timeout wait
      while (m_writeBlock && this->isFull())
	{
	  if (m_writeTimeout < 0) 
	    {
	      usleep(TIMEOUT_TICK_USEC);
	      continue;
	    }

	  // timeout wait
	  ACE_Time_Value tt;
	  tt = ACE_OS::gettimeofday();
	  tm_cur = tt.operator timeval();
	  long int sec (tm_cur.tv_sec  - tm_pre.tv_sec);
	  long int usec(tm_cur.tv_usec - tm_pre.tv_usec);

	  timeout -= (sec * USEC_PER_SEC + usec);
	  if (timeout < 0) {
	    status = BUF_TIMEOUT;
	    break;
	  }
	  tm_pre = tm_cur;
	  usleep(TIMEOUT_TICK_USEC);
	}

      if (this->isFull() && m_OnOverflow != NULL)
	{
	  (*m_OnOverflow)(value);
	  return BUF_NO_BUF;
	}

      if (m_OnWriteConvert == NULL) 
	{
	  status = this->put(value);
	}
      else
	{
	  status = this->put((*m_OnWriteConvert)(value));
	}
      return status;
    }
    

    /*!
     * @if jp
     *
     * @brief DataPort ͤɤ߽Ф
     *
     * DataPort ͤɤ߽Ф
     *
     * - Хåե󥯥 OnRead åȤƤ硢
     *   DataPort ݻХåեɤ߽Ф OnRead ƤФ롣
     * - DataPort ݻХåեե򸡽ФǤХåեǡ
     *   ġɤ߽Фݤ˥Хåեե򸡽Ф硢
     *   Хåե󥯥 OnUnderflow ƤФ롣
     * - Хåե󥯥 OnReadConvert åȤƤ硢
     *   Хåե񤭹߻ˡOnReadConvert  operator()() ͤ
     *   read()ͤȤʤ롣
     * - setReadTimeout() ˤɤ߽ФΥॢȤꤵƤ硢
     *   Хåեե֤ޤǥॢȻ֤Ԥ
     *   OnUnderflowåȤƤФƤӽФ
     *
     * @else
     *
     * @brief [CORBA interface] Put data on InPort
     *
     * @endif
     */
    int read(DataType& value)
    {
      int status = BUF_SUCCESS;

      // std::cout << "InPort::read(const DataType& value)" << std::endl;

      if (m_OnRead != NULL) (*m_OnRead)();      

      long int timeout = m_readTimeout;

      timeval tm_cur, tm_pre;
      ACE_Time_Value tt;
      tt = ACE_OS::gettimeofday();
      tm_pre = tt.operator timeval();

      // blocking and timeout wait
      while (m_readBlock && this->isEmpty())
	{
	  if (m_readTimeout < 0)
	    {
	      usleep(TIMEOUT_TICK_USEC);
	      continue;
	    }

	  // timeout wait
	  ACE_Time_Value tt;
	  tt = ACE_OS::gettimeofday();
	  tm_cur = tt.operator timeval();
	  long int sec (tm_cur.tv_sec  - tm_pre.tv_sec);
	  long int usec(tm_cur.tv_usec - tm_pre.tv_usec);

	  timeout -= (sec * USEC_PER_SEC + usec);
	  if (timeout < 0) {
	    status = BUF_TIMEOUT;
	    break;
	  }
	  tm_pre = tm_cur;
	  usleep(TIMEOUT_TICK_USEC);
	}

      if (this->isEmpty() && m_OnUnderflow != NULL)
	{
	  value = (*m_OnUnderflow)();
	  return status;
	}

      if (m_OnReadConvert == NULL) 
	{
	  status = this->get(value);
	  //	  std::cout << "hoge" << std::endl;
	}
      else
	{
	  status = this->get(value);
	  m_value = (*m_OnReadConvert)(value);
	  value = m_value;
	}
      return status;
    }



    /*!
     * @if jp
     *
     * @brief InPort Υ󥰥Хåեͤ
     *
     * InPort Υ󥰥Хåեͤ롣
     *
     * @else
     *
     * @brief Initialize ring buffer value of InPort
     *
     * @endif
     */
    virtual void init(DataType& value)
    {
      //      m_buffer.init(value);
    }
    
    
    /*!
     * @if jp
     *
     * @brief Хɤ줿 T ѿ InPort Хåեκǿͤɤ߹
     *
     * Хɤ줿 T Υǡ InPort κǿͤɤ߹ࡣ
     * 󥹥ȥ饯 T ѿ InPort ХɤƤʤФʤʤ
     * Υ᥽åɤϥݥ⡼եå˻ѤȤƤ뤿ᡢ
     * ˰¸ʤͤȤʤäƤ롣
     *
     * @else
     *
     * @brief Read into bound T-type data from current InPort
     *
     * @endif
     */
    /* do not use those functions KEK
    void update()
    {
      try
	{
	  int status = this->get(m_value);
	}
      catch (...)
	{
	  if (m_OnUnderflow != NULL) (*m_OnUnderflow)();
	}
      return;
    };
    */
    
    /*!
     * @if jp
     *
     * @brief T Υǡ InPort κǿͥǡɤ߹
     *
     * @param rhs InPort Хåեͤɤ߹ T ѿ
     *
     * @else
     *
     * @brief Read from InPort buffer to type-T variable
     *
     * @param rhs type-T variable which is wrote from InPort buffer
     *
     * @endif
     */
    /* do not use those functions

    void operator>>(DataType& rhs)
    {
      int status = read(rhs);
      return;
    }
    

    void operator<<(DataType& value)
    {
      int status = write(value);
      return;
    }
    */
    
    /*!
     * @if jp
     *
     * @brief ̤ɤοǡ
     *
     * @else
     *
     * @brief Get number of new data to be read.
     *
     * @endif
     */
    /*
    virtual int getNewDataLen()
    {
      return m_buffer->new_data_len();
    }
    */    
    
    /*!
     * @if jp
     *
     * @brief ̤ɤοǡ
     *
     * @else
     *
     * @brief Get new data to be read.
     *
     * @endif
     */
    /*
    virtual std::vector<T> getNewList()
    {
      return m_buffer.get_new_list();
    }
    */
    
    /*!
     * @if jp
     *
     * @brief ̤ɤοǡս(->)Ǽ
     *
     * @else
     *
     * @brief Get new data to be read.
     *
     * @endif
     */
    /*
    virtual std::vector<T> getNewListReverse()
    {
      return m_buffer.get_new_rlist();
    }
    */

    /*!
     * @if jp
     *
     * @brief InPort Хåե˥ǡϻΥХå
     *
     * InPort ĥХåե˥ǡput줿Ȥ˸ƤФ륳Хå
     * ֥Ȥꤹ롣ꤵ륳Хå֥Ȥ
     * InPort<DataType>::OnPut饹Ѿ const DataType& 
     *  void  operator() ؿƤɬפ롣
     *
     * struct MyOnPutCallback : public InPort<DataType> {<br>
     *   void operator()(const DataType data) {<br>
     *     <br>
     *   }<br>
     * };<br>
     * Τ褦˥Хå֥Ȥ<br>
     * <br> 
     * m_inport.setOnPut(new MyOnPutCallback());<br>
     * Τ褦˥Хå֥Ȥ򥻥åȤ롣
     *
     * @else
     *
     * @brief Get new data to be read.
     *
     * @endif
     */
    inline void setOnWrite(OnWrite<DataType>* on_write)
    {
      m_OnWrite = on_write;
    }

    inline void setOnWriteConvert(OnWriteConvert<DataType>* on_wconvert)
    {
      m_OnWriteConvert = on_wconvert;
    }

    inline void setOnRead(OnRead<DataType>* on_read)
    {
      m_OnRead = on_read;
    }

    inline void setOnReadConvert(OnReadConvert<DataType>* on_rconvert)
    {
      m_OnReadConvert = on_rconvert;
    }

    inline void setOnOverflow(OnOverflow<DataType>* on_overflow)
    {
      m_OnOverflow = on_overflow;
    }

    inline void setOnUnderflow(OnUnderflow<DataType>* on_underflow)
    {
      m_OnUnderflow = on_underflow;
    }


  private:
    /*!
     * @if jp
     * @brief ݡ̾
     * @else
     * @berif Port's name
     * @else
     */
    std::string m_name;

    /*!
     * @if jp
     * @brief Хɤ T ѿؤλ
     * @else
     * @brief reference to type-T value bound this OutPort
     * @endif
     */
    DataType& m_value;
    
    /*!
     * @if jp
     * @brief Хåե
     * @else
     * @brief Buffer
     * @endif
     */
    //    Buffer<DataType> m_buffer;

    bool m_readBlock;
    long int m_readTimeout;
    bool m_writeBlock;
    long int m_writeTimeout;

    /*!
     * @if jp
     * @brief OnWrite Хåե󥯥ؤΥݥ
     * @else
     * @brief OnWrite callback functor pointer
     * @endif
     */
    OnWrite<DataType>* m_OnWrite;

    /*!
     * @if jp
     * @brief OnWriteConvert Хåե󥯥ؤΥݥ
     * @else
     * @brief OnWriteConvert callback functor pointer
     * @endif
     */
    OnWriteConvert<DataType>* m_OnWriteConvert;

    /*!
     * @if jp
     * @brief OnRead Хåե󥯥ؤΥݥ
     * @else
     * @brief OnRead callback functor pointer
     * @endif
     */
    OnRead<DataType>* m_OnRead;

    /*!
     * @if jp
     * @brief OnReadConvert Хåե󥯥ؤΥݥ
     * @else
     * @brief OnReadConvert callback functor pointer
     * @endif
     */
    OnReadConvert<DataType>* m_OnReadConvert;

    /*!
     * @if jp
     * @brief OnOverflow Хåե󥯥ؤΥݥ
     * @else
     * @brief OnOverflow callback functor pointer
     * @endif
     */
    OnOverflow<DataType>* m_OnOverflow;

    /*!
     * @if jp
     * @brief OnUnderflow Хåե󥯥ؤΥݥ
     * @else
     * @brief OnUnderflow callback functor pointer
     *
     * @endif
     */
    OnUnderflow<DataType>* m_OnUnderflow;

    
  };
  
}; // End of namesepace RTM

#endif // RtcInPort_h
