// -*- C++ -*-
/*!
 * @file  InPortCorbaConsumer.h
 * @brief InPortCorbaConsumer class
 * @date  $Date: 2007/09/19 07:47:28 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: InPortCorbaConsumer.h,v 1.3.4.2 2007/09/19 07:47:28 n-ando Exp $
 *
 */

/*
 * $Log: InPortCorbaConsumer.h,v $
 * Revision 1.3.4.2  2007/09/19 07:47:28  n-ando
 * A trivial fix.
 *
 * Revision 1.3.4.1  2007/08/20 06:31:10  n-ando
 * push() function was modified to detect buffer read error.
 *
 * Revision 1.3  2007/01/09 09:56:38  n-ando
 * Kanji-code has changed.
 *
 * Revision 1.2  2007/01/06 17:50:57  n-ando
 * Interface subscription/unsubscription functions (subscribeInterface()
 * and unsubscribeInterface()) are added.
 * The clone() function to clone the instance is added.
 *
 * Revision 1.1  2006/12/02 18:39:57  n-ando
 * InPortCorbaConsumer class was moved from InPortConsumer.h
 *
 */

#ifndef InPortCorbaConsumer_h
#define InPortCorbaConsumer_h

#include <rtm/idl/DataPortSkel.h>
#include <rtm/BufferBase.h>
#include <rtm/CorbaConsumer.h>
#include <rtm/InPortConsumer.h>
#include <iostream>
#include <unistd.h>
 
namespace RTC
{
  template <class DataType>
  class InPortCorbaConsumer
    : public InPortConsumer,
      public CorbaConsumer<RTC::InPortAny>
  {
  public:
    InPortCorbaConsumer(BufferBase<DataType>& buffer)
      : m_buffer(buffer)
    {
    }


    InPortCorbaConsumer(const InPortCorbaConsumer<DataType>& consumer)
      : CorbaConsumer<RTC::InPortAny>(consumer), m_buffer(consumer.m_buffer)
    {
    }

    InPortCorbaConsumer&
    operator=(const InPortCorbaConsumer<DataType>& consumer)
    {
      if (this == &consumer) return *this;
      m_buffer = consumer.m_buffer;
    }


    virtual ~InPortCorbaConsumer()
    {}

    void put(DataType& data)
    {
      CORBA::Any tmp;
      tmp <<= data;
      _ptr()->put(tmp);
    }

    int push()
    {
      DataType data;
      CORBA::Long status;
      CORBA::Any tmp;
      int st = m_buffer.read(data);
      if(st)
	return st; // timeout or fatal
      tmp <<= data;

      // ϥ顼򤹤٤
      if (CORBA::is_nil(_ptr())) return -1; // fatal
      for(int i = 0; i < 100; i++) { // retry count = 100
	try
	  {
	    status = _ptr()->put(tmp);
	    st = static_cast<int>(status);
	  }
	catch(...)
	  {
	    //hoge ֥Ȥ̵ˤʤädisconnect٤
	    st = -1;
	  }
	if(st == 1 ) {
	  std::cout << "InPortCorbaConsumer::push: timeout occurred... count = " << i << std::endl;
	  usleep(0); // wait for next operation
	}
	if(st != 1) break;  // BUF_TIMEOUT = 1
	if( i == 99 )
	  st = -1; //fatal
      }
      return st;
    }

    virtual InPortCorbaConsumer* clone() const
    {
      return new InPortCorbaConsumer<DataType>(*this);
    }

    virtual bool subscribeInterface(const SDOPackage::NVList& properties)
    {
      if (!NVUtil::isStringValue(properties,
				 "dataport.dataflow_type",
				 "Push"))
	{
	  return false;
	}
      
      CORBA::Long index;
      index = NVUtil::find_index(properties,
				 "dataport.corba_any.inport_ref");
      if (index < 0)
	{
	  return false;
	}
      
      CORBA::Object_ptr obj;
      if (properties[index].value >>= CORBA::Any::to_object(obj))
	{
	  if (!CORBA::is_nil(obj))
	    {
	      setObject(obj);
	      return true;
	    }
	}
      return false;;
    }


    virtual void unsubscribeInterface(const SDOPackage::NVList& properties)
    {
      ;
    }


  private:
    BufferBase<DataType>& m_buffer;
  };
};     // namespace RTC
#endif // InPortCorbaConsumer_h

