// -*- C++ -*-
/*!
 * @file Manager.h
 * @brief RTComponent manager class
 * @date $Date: 2007/07/20 15:56:14 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: Manager.h,v 1.9.2.1 2007/07/20 15:56:14 n-ando Exp $
 *
 */

/*
 * $Log: Manager.h,v $
 * Revision 1.9.2.1  2007/07/20 15:56:14  n-ando
 * Functor InstanceName was modified for win32 porting.
 *
 * Revision 1.9  2007/04/26 15:36:54  n-ando
 * The header include order was modified to define _REENTRANT before
 * including ace/config-lite.h in Linux systems.
 * In ace 5.4.7 or later, _REENTRANT flag should be defined explicitly.
 *
 * Revision 1.8  2007/04/23 04:53:29  n-ando
 * Component instantiation processes were divided into some functions.
 *
 * Revision 1.7  2007/04/17 09:22:08  n-ando
 * Namespace of Timer class was changed from ::Timer to RTC::Timer.
 *
 * Revision 1.6  2007/04/13 18:02:14  n-ando
 * Some configuration properties handling processes were changed.
 *
 * Revision 1.5  2007/01/21 09:54:30  n-ando
 * A trivial bug fix.
 *
 * Revision 1.4  2007/01/14 19:42:37  n-ando
 * The activate() function now performs POA manager activation and
 * invoking ModuleInitProc.
 * Debugging messages are now output to system logger.
 *
 * Revision 1.3  2006/11/06 01:31:57  n-ando
 * Some Manager's functions has been implemented.
 * - Component creation process
 * - System logger initialization
 *
 * Revision 1.2  2006/10/25 17:28:05  n-ando
 * Component factory registration and relative functions are implemented.
 *
 * Revision 1.1  2006/10/17 10:21:24  n-ando
 * The first commitment.
 *
 *
 */

#ifndef Manager_h
#define Manager_h

#include <rtm/RTC.h>

#include <iostream>
#include <string>
#include <vector>

#include <ace/Synch.h>
#include <ace/Task.h>

#include <rtm/Factory.h>
#include <rtm/ECFactory.h>
#include <rtm/ObjectManager.h>
#include <rtm/RTObject.h>
#include <rtm/SystemLogger.h>

class Properties;


namespace RTC
{
  class CorbaNaming;
  class ModuleManager;
  class NamingManager;
  class CorbaObjectManager;
  class Manager;
  class Timer;

  typedef void (*ModuleInitProc)(Manager* manager);  

  class Manager
  {
  protected:
    /*!
     * @if jp
     * @brief Protected 󥹥ȥ饯
     * @else
     * @brief Protected Constructor
     * @endif
     */
    Manager();


    /*!
     * @if jp
     * @brief Protected ԡ󥹥ȥ饯
     * @else
     * @brief Protected Copy Constructor
     * @endif
     */
    Manager(const Manager& manager);
      //      Manager& operator=(const Manager& manager){return manager;};
    
  public:
    /*!
     * @if jp
     * @brief ޥ͡ν
     *
     * ޥ͡ static дؿ
     * ޥ͡򥳥ޥɥ饤Ϳƽ롣
     * ޥ͡Ѥϡɬνдؿ init() 
     * ƤФʤФʤʤ
     * ޥ͡Υ󥹥󥹤ˡȤơinit(), instance() 
     * 2Ĥ static дؿѰդƤ뤬init()Ǥ
     * ԤʤᡢManager ¸֤ΰֺǽˤinit()Ƥɬפ롣
     *
     * ޥ͡ν
     * - initManager: configեɤ߹ߡ֥ƥ
     * - initLogger: Logger
     * - initORB: ORB 
     * - initNaming: NamingService 
     * - initExecutionContext: ExecutionContext factory 
     * - initTimer: Timer 
     *
     * @param argc ޥɥ饤ο
     * @param argv ޥɥ饤
     * 
     * @else
     * @brief Initializa manager
     *
     * This is the static member function to tintialize the Manager.
     * The Manager is initialized by given arguments.
     * At the starting the manager, this static member function "must" be
     * called from application program. The manager has two static functions
     * to get the instance, "init()" and "instance()". Since initializing
     * process is only performed by the "init()" function, the "init()" has
     * to be called at the beginning of the lifecycle of the Manager.
     * function.
     *
     * @param argc The number of command line argument. 
     * @param argv The array of the command line arguments.
     *
     * @endif
     */ 
    static Manager* init(int argc, char** argv);


    /*!
     * @if jp
     * @brief ޥ͡Υ󥹥󥹤μ
     *
     * ޥ͡Υ󥹥󥹤 static дؿ
     * δؿƤˡɬνдؿ init() ƤФƤ
     * ɬפ롣
     *
     * @return Manager ͣΥ󥹥󥹤λ
     * 
     * @else
     *
     * @brief Get instance of the manager
     *
     * This is the static member function to get the instance of the Manager.
     * Before calling this function, ensure that the initialization function
     * "init()" is called.
     *
     * @return The only instance reference of the manager
     *
     * @endif
     */ 
    static Manager& instance();

    //============================================================
    // Manager
    //============================================================



    void terminate();
    void shutdown();
    void join();
    
    Logbuf& getLogbuf() {return m_Logbuf;}
    Properties& getConfig() { return m_config;}
    
    
    
    
    /*!
     * @if jp
     *
     * @brief ץΥå
     *
     * Υڥ졼ϥ桼Ԥ⥸塼νץ
     * ꤹ롣ꤵ줿ץϡޥ͡㤬졢
     * ƥֲ줿塢Ŭڤʥߥ󥰤Ǽ¹Ԥ롣
     *
     * @param proc ץδؿݥ
     *
     * @else
     *
     * @brief Run the Manager
     *
     * This operation sets the initial procedure call to process module
     * initialization, other user defined initialization and so on.
     * The given procedure will be called at the proper timing after the 
     * manager initialization, activation and run.
     *
     * @param proc A function pointer to the initial procedure call
     *
     * @endif
     */
    void setModuleInitProc(ModuleInitProc proc);


    /*!
     * @if jp
     *
     * @brief ManagerΥƥֲ
     *
     * Υڥ졼ϰʲνԤ
     * - CORBA POAManager Υƥֲ
     * - ޥ͡CORBA֥ȤΥƥֲ
     * - Manager ֥ȤؤΥ֥ȻȤϿ
     *
     * Υڥ졼ϡޥ͡ν塢runManager()
     * ˸Ƥɬפ롣
     *
     * @else
     *
     * @brief Activate Manager
     *
     * This operation do the following,
     * - Activate CORBA POAManager
     * - Activate Manager CORBA object
     * - Bind object reference of the Manager to the nameserver
     *
     * This operationo should be invoked after Manager:init(),
     * and before tunManager().
     *
     * @endif
     */
    bool activateManager();

    /*!
     * @if jp
     *
     * @brief Managerμ¹
     *
     * Υڥ졼ϥޥ͡Υᥤ롼פ¹Ԥ롣
     * Υᥤ롼ǤϡCORBA ORBΥ٥ȥ롼
     * 롣ǥեȤǤϡΥڥ졼ϥ֥å
     * Manager::destroy() ƤФޤǽᤵʤ
     *  no_block  true ꤵƤϡǥ٥ȥ롼
     * 륹åɤư֥å˽᤹
     *
     * @param no_block false: ֥å󥰥⡼, true: Υ֥å󥰥⡼
     *
     * @else
     *
     * @brief Run the Manager
     *
     * This operation processes the main event loop of the Manager.
     * In this main loop, CORBA's ORB event loop or other processes
     * are performed. As the default behavior, this operation is going to
     * blocking mode and never returns until manager::destroy() is called.
     * When the given argument "no_block" is set to "true", this operation
     * creates a thread to process the event loop internally, and it doesn't
     * block and returns.
     *
     * @param no_block false: Blocking mode, true: non-blocking mode.
     *
     * @endif
     */
    void runManager(bool no_block = false);
    



    //============================================================
    // Module management
    //============================================================
    /*!
     * @if jp
     * @brief [CORBA interface] ⥸塼Υ
     *
     * ݡͥȤΥ⥸塼ɤƽؿ¹Ԥ롣
     *
     * @param fname   ⥸塼ե̾
     * @param initfunc ؿ̾
     * 
     * @else
     *
     * @brief [CORBA interface] Load module
     *
     * Load module (shared library, DLL etc..) by file name,
     * and invoke initialize function.
     *
     * @param fname    The module file name
     * @param initfunc The initialize function name
     *
     * @endif
     */  
    void load(const char* fname, const char* initfunc);


    /*!
     * @if jp
     *
     * @brief ⥸塼Υ
     *
     * ⥸塼򥢥ɤ
     *
     * @param fname ⥸塼Υե̾
     * 
     * @else
     *
     * @brief Unload module
     *
     * Unload shared library.
     *
     * @param pathname Module file name
     *
     * @endif
     */ 

    void unload(const char* fname);


    /*!
     * @if jp
     *
     * @brief ⥸塼Υ
     *
     * ⥸塼򤹤٤ƥɤ
     *
     * @else
     *
     * @brief Unload module
     *
     * Unload all loaded shared library.
     *
     * @endif
     */ 
    void unloadAll();


    /*!
     * @if jp
     * @brief ɺѤߤΥ⥸塼ꥹȤ
     * @else
     * @brief Get loaded module names
     * @endif
     */
    std::vector<std::string> getLoadedModules();
    

    /*!
     * @if jp
     * @brief ɲǽʥ⥸塼ꥹȤ
     * @else
     * @brief Get loadable module names
     * @endif
     */
    std::vector<std::string> getLoadableModules();

    //============================================================
    // Component Factory Management
    //============================================================
    /*!
     * @if jp
     * @brief RTݡͥȥեȥϿ
     * @else
     * @brief Register RT-Component Factory
     * @endif
     */
    bool registerFactory(Properties& profile,
			 RtcNewFunc new_func,
			 RtcDeleteFunc delete_func);
    
    bool registerECFactory(const char* name,
			   ECNewFunc new_func,
			   ECDeleteFunc delete_func);
    
    /*!
     * @if jp
     * @brief եȥꥹȤ
     * @else
     * @brief Get the list of all RT-Component Factory
     * @endif
     */
    std::vector<std::string> getModulesFactories();
    
    //============================================================
    // Component management
    //============================================================
    /*!
     * @if jp
     * @brief RTݡͥȤ
     * @else
     * @brief Create RT-Component
     * @endif
     */
    RtcBase* createComponent(const char* module_name);

    void cleanupComponent(RtcBase* comp);

    /*!
     * @if jp
     * @brief RTݡͥȤľ Manager Ͽ
     * @else
     * @brief Register RT-Component directly without Factory
     * @endif
     */
    bool registerComponent(RtcBase* comp);

    bool unregisterComponent(RtcBase* comp);

    bool bindExecutionContext(RtcBase* comp);

    /*!
     * @if jp
     * @brief Manager ϿƤRTݡͥȤ
     * @else
     * @brief Unregister RT-Component that is registered in the Manager
     * @endif
     */
    void deleteComponent(const char* instance_name);
    
    /*!
     * @if jp
     * @brief Manager ϿƤRTݡͥȤ
     * @else
     * @brief Get RT-Component's pointer
     * @endif
     */
    RtcBase* getComponent(const char* instance_name);
    
    /*!
     * @if jp
     * @brief Manager ϿƤRTݡͥȤ
     * @else
     * @brief Get all RT-Component's pointer
     * @endif
     */
    std::vector<RtcBase*> getComponents();
    

    //============================================================
    // CORBA Ϣ
    //============================================================
    /*!
     * @if jp
     * @brief ORB Υݥ󥿤
     * @else
     * @brief Get the pointer to the ORB
     * @endif
     */
    CORBA::ORB_ptr getORB();

    /*!
     * @if jp
     * @brief Manager  RootPOA Υݥ󥿤
     * @else
     * @brief Get the pointer to the RootPOA 
     * @endif
     */
    PortableServer::POA_ptr getPOA();
    PortableServer::POAManager_ptr getPOAManager();


    //============================================================
    // Protected functions
    //============================================================
  protected:
    
    //============================================================
    // Manager initialize and finalization
    //============================================================
    /*!
     * @if jp
     * @brief Manager 
     * @else
     * @brief Manager internal initialization
     * @endif
     */
    void initManager(int argc, char** argv);


    /*!
     * @if jp
     * @brief Manager νλ
     * @else
     * @brief Manager internal finalization
     * @endif
     */
    void shutdownManager();


    //============================================================
    // Logger initialize and terminator
    //============================================================
    /*!
     * @if jp
     * @brief System logger ν
     * @else
     * @brief System logger initialization
     * @endif
     */
    bool initLogger();


    /*!
     * @if jp
     * @brief System Logger νλ
     * @else
     * @brief System Logger finalization
     * @endif
     */
    void shutdownLogger();


    //============================================================
    // ORB initialization and finalization
    //============================================================
    /*!
     * @if jp
     * @brief CORBA ORB ν
     * @else
     * @brief CORBA ORB initialization
     * @endif
     */
    bool initORB();
    
    
    /*!
     * @if jp
     * @brief ORB Υޥɥ饤󥪥ץ
     * @else
     * @brief ORB command option creation
     * @endif
     */
    std::string createORBOptions();
    
    
    /*!
     * @if jp
     * @brief ORB νλ
     * @else
     * @brief ORB finalization
     * @endif
     */
    void shutdownORB();

   
    //============================================================
    // NamingService initialization and finalization
    //============================================================
    /*!
     * @if jp
     * @brief NamingManager ν
     * @else
     * @brief NamingManager initialization
     * @endif
     */
    bool initNaming();
    void shutdownNaming();


    //============================================================
    // Component management
    //============================================================
    void shutdownComponents();


    void configureComponent(RtcBase* comp);


    bool initExecContext();

    bool initTimer();

    bool mergeProperty(Properties& prop, const char* file_name);
    std::string formatString(const char* naming_format,
			     Properties& prop);
    
    //============================================================
    // protected ѿ
    //============================================================

    //------------------------------------------------------------
    // static var
    //------------------------------------------------------------
    /*!
     * @if jp
     * @brief ͣ Manager ؤΥݥ
     * @else
     * @brief The pointer to the Manager
     * @endif
     */
    static Manager* manager;

    /*!
     * @if jp
     * @brief ͣ Manager ؤΥݥ󥿤Ф mutex
     * @else
     * @brief The mutex of the pointer to the Manager 
     * @endif
     */
    static ACE_Thread_Mutex mutex;
    

    //------------------------------------------------------------
    // CORBA var
    //------------------------------------------------------------
    /*!
     * @if jp
     * @brief ORB ؤΥݥ
     * @else
     * @brief The pointer to the ORB
     * @endif
     */
    CORBA::ORB_var m_pORB;

    /*!
     * @if jp
     * @brief POA ؤΥݥ
     * @else
     * @brief The pointer to the POA
     * @endif
     */
    PortableServer::POA_var m_pPOA;

    /*!
     * @if jp
     * @brief POAManager ؤΥݥ
     * @else
     * @brief The pointer to the POAManager
     * @endif
     */
    PortableServer::POAManager_var m_pPOAManager;


    //------------------------------------------------------------
    // Manager's variable
    //------------------------------------------------------------
    /*!
     * @if jp
     * @brief 桼ؿؤΥݥ
     * @else
     * @brief User's initialization function's pointer
     * @endif
     */
    ModuleInitProc m_initProc;

    /*!
     * @if jp
     * @brief Manager  configuration Ǽ Properties
     * @else
     * @brief Managaer's configuration Properties
     * @endif
     */
    Properties m_config;

    /*!
     * @if jp
     * @brief ModuleManager ؤΥݥ
     * @else
     * @brief The pointer to the ModuleManager
     * @endif
     */
    ModuleManager* m_module;
    
    /*!
     * @if jp
     * @brief ModuleManager ؤΥݥ
     * @else
     * @brief The pointer to the ModuleManager
     * @endif
     */
    NamingManager* m_namingManager;

    CorbaObjectManager* m_objManager;

    /*!
     * @if jp
     * @brief Timer Object
     * @else
     * @brief Timer Object
     * @endif
     */
    Timer* m_timer;


    //------------------------------------------------------------
    // Logger
    //------------------------------------------------------------
    /*!
     * @if jp
     * @brief Хåե
     * @else
     * @brief Logger buffer
     * @endif
     */
    Logbuf m_Logbuf;

    /*!
     * @if jp
     * @brief Хåե
     * @else
     * @brief Logger mediation buffer
     * @endif
     */
    MedLogbuf m_MedLogbuf;

    /*!
     * @if jp
     * @brief ȥ꡼
     * @else
     * @brief Logger stream
     * @endif
     */
    LogStream rtcout;
    

    //============================================================
    // ݡͥȥޥ͡
    //============================================================
    // ObjectManager ϤҸ쥯饹
    struct InstanceName
    {
      InstanceName(RtcBase* comp) : m_name(comp->getInstanceName()) {};
      InstanceName(const char* name) : m_name(name) {};
      InstanceName(const std::string name) : m_name(name) {};
      bool operator()(RtcBase* comp)
      {
	return m_name == comp->getInstanceName();
      }
      std::string m_name;
    };

    typedef ObjectManager<std::string,
			  RtcBase,
			  InstanceName> ComponentManager;
    /*!
     * @if jp
     * @brief ݡͥȥޥ͡
     * @else
     * @brief The ComponentManager
     * @endif
     */
    ComponentManager m_compManager;


    //============================================================
    // ݡͥȥեȥ
    //============================================================
    // ݡͥȥեȥϤҸ쥯饹
    struct FactoryPredicate
    {
      FactoryPredicate(const char* name) : m_name(name){};
      FactoryPredicate(FactoryBase* factory)
	: m_name(factory->profile()["implementation_id"]) {};
      bool operator()(FactoryBase* factory)
      {
	return m_name == factory->profile()["implementation_id"];
      }
      std::string m_name;
    };
    /*!
     * @if jp
     * @brief ݡͥȥեȥ
     * @else
     * @brief The ComponentFactory
     * @endif
     */
    typedef ObjectManager<const char*,
			  FactoryBase,
			  FactoryPredicate> FactoryManager;
    /*!
     * @if jp
     * @brief ComponentManager
     * @else
     * @brief The ComponentManager
     * @endif
     */
    FactoryManager m_factory;

    //============================================================
    // ExecutionContextեȥ
    //============================================================
    // ECեȥϤҸ쥯饹
    struct ECFactoryPredicate
    {
      ECFactoryPredicate(const char* name) : m_name(name){};
      ECFactoryPredicate(ECFactoryBase* factory)
	: m_name(factory->name()) {};
      bool operator()(ECFactoryBase* factory)
      {
	return m_name == factory->name();
      }
      std::string m_name;
    };
    typedef ObjectManager<const char*,
			  ECFactoryBase,
			  ECFactoryPredicate> ECFactoryManager;
    ECFactoryManager m_ecfactory;
    std::vector<ExecutionContextBase*> m_ecs;


    // եȥ̾ꥹȥåפ뤿Υե󥯥
    struct ModuleFactories
    {
      void operator()(FactoryBase* f)
      {
	modlist.push_back(f->profile().getProperty("implementation_id"));
      }
      std::vector<std::string> modlist;
    };


    //------------------------------------------------------------
    // ORB runner
    //------------------------------------------------------------
    class OrbRunner
      : public ACE_Task<ACE_MT_SYNCH>
    {
    public:
      OrbRunner(CORBA::ORB_ptr orb) : m_pORB(orb)
      {
	open(0);
      };
      virtual int open(void *args)
      {
	activate();
	return 0;
      }
      virtual int svc(void)
      {
	m_pORB->run();
	Manager::instance().shutdown();
	return 0;
      }
      virtual int close(unsigned long flags)
      {
	return 0;
      }
    private:
      CORBA::ORB_ptr m_pORB;
      
    };
    OrbRunner* m_runner;


    //------------------------------------------------------------
    // Manager Terminator
    //------------------------------------------------------------
    class Terminator
      : public ACE_Task<ACE_MT_SYNCH>
    {
    public:
      Terminator(Manager* manager) : m_manager(manager) {};
      void terminate()
      {
	open(0);
      }
      virtual int open(void *args)
      {
	activate();
	return 0;
      }
      virtual int svc(void)
      {
	Manager::instance().shutdown();
	return 0;
      }
      Manager* m_manager;
    };

    Terminator* m_terminator;

    struct Term
    {
      int waiting;
      ACE_Thread_Mutex mutex;
    };
    Term m_terminate;



  }; // class Manager
}; // namespace RTC

#endif // Manager_h
