// -*- C++ -*-
/*!
 * @file ManagerConfig.h
 * @brief RTC manager configuration
 * @date $Date: 2007/04/13 18:02:28 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2003-2005
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: ManagerConfig.h,v 1.4 2007/04/13 18:02:28 n-ando Exp $
 *
 */

/*
 * $Log: ManagerConfig.h,v $
 * Revision 1.4  2007/04/13 18:02:28  n-ando
 * Some configuration properties handling processes were changed.
 *
 * Revision 1.3  2006/11/06 01:26:28  n-ando
 * Some trivial fixes.
 *
 * Revision 1.2  2006/10/23 08:38:23  n-ando
 * To get Property after "init()" calling, getConfig() was added.
 *
 * Revision 1.1  2006/10/17 10:21:38  n-ando
 * The first commitment.
 *
 *
 */


#ifndef ManagerConfig_h
#define ManagerConfig_h

#include <rtm/Properties.h>


namespace RTC
{

  
  /*!
   * @if jp
   *
   * @class ManagerConfig
   * @brief Manager configuration 饹
   *
   * Manager Υե졼Ԥޥɥ饤ꡢ
   * (뤤ϰʤ)󥹥󥹲롣Manager Υץѥƥ
   * Ԥ
   *
   * (ե)λζϰʲΤȤǤ롣
   * 夬äȤ⶯äȤ夤
   * <OL>
   * <LI>ޥɥ饤󥪥ץ "-f"
   * <LI>Ķѿ "RTC_MANAGER_CONFIG"
   * <LI>ǥեե "./rtc.conf"
   * <LI>ǥեե "/etc/rtc.conf"
   * <LI>ǥեե "/etc/rtc/rtc.conf"
   * <LI>ǥեե "/usr/local/etc/rtc.conf"
   * <LI>ǥեե "/usr/local/etc/rtc/rtc.conf"
   * <LI>ߥե졼
   *</OL>
   * ޥɥ饤󥪥ץ "-d" ꤵ줿ϡ
   * (Ȥ -f եꤷƤ)ߥե졼
   * Ѥ롣
   *
   * @else
   *
   * @brief
   *
   * @endif
   */
  class ManagerConfig
  {
  public:
    // The list of default configuration file path.
    static const char* config_file_path[];
    
    // Environment value to specify configuration file
    static const char* config_file_env;


    /*!
     * @if jp
     *
     * @brief ManagerConfig 󥹥ȥ饯
     *
     * ⤷ʤ󥹥ȥ饯
     *
     * @else
     *
     * @brief ManagerConfig constructor
     *
     * Do nothing. 
     *
     * @endif
     */
    ManagerConfig();


    /*!
     * @if jp
     *
     * @brief ManagerConfig 󥹥ȥ饯
     *
     * Ϳ줿ˤƱˤ륳󥹥ȥ饯
     *
     * @param argc ޥɥ饤ο
     * @param argv ޥɥ饤
     *
     * @else
     *
     * @brief ManagerConfig constructor
     *
     * The constructor that performs initialization at the same time with
     * given arguments.
     *
     * @param argc The number of command line arguments
     * @param argv The command line arguments
     *
     * @endif
     */
    ManagerConfig(int argc, char** argv);


    /*!
     * @if jp
     *
     * @brief ManagerConfig ǥȥ饯
     *
     * @else
     *
     * @brief ManagerConfig destructor
     *
     * @endif
     */
    virtual ~ManagerConfig();
    

    /*!
     * @if jp
     *
     * @brief 
     *
     * ǥեȥѥ饳ե졼եõ
     * ԤȤȤ˥ץѥƥ֤ѥ˥ե졼
     * ե뤬¸ߤʤ硢ǥեȤΥե졼
     * ֤
     *
     * @return 줿 Property 
     *
     * @else
     *
     * @brief Initialization
     *
     * This operation searches the configuration file from default search path,
     * and initialize and return default properties.
     * If there is no configuration file in the default search path,
     * default configuration statically defined is used.
     *
     * @return Initialized Property value.
     *
     * @endif
     */
    //    void init();



    /*!
     * @if jp
     *
     * @brief 
     *
     * ޥɥ饤Ϳƽ롣ޥɥ饤󥪥ץ
     * ʲΤΤѲǽǤ롣
     *
     * -f file   : ե졼եꤹ롣<br>
     * -l module : ɤ⥸塼ꤹ롣<br>
     * -o options: ¾ץꤹ롣<br>
     * -d        : ǥեȤΥե졼Ȥ<br>
     *
     * @else
     *
     * @brief Initialization
     *
     * Initialize with command line options. The following command options
     * are available.
     *
     * -f file   : Specify a configuration file. <br>
     * -l module : Specify modules to be loaded at the beginning. <br>
     * -o options: Other options. <br>
     * -d        : Use default static configuration. <br>
     *
     * @endif
     */
    void init(int argc, char** argv);


    /*!
     * @if jp
     * @brief Configuration η̤PropertyȿǤ
     * @else
     * @brief Apply configuration results to Property
     * @endif
     */
    void configure(Properties& prop);


    /*!
     * @if jp
     *
     * @brief ե졼
     *
     * ե졼롣init()ƤӽФ˸Ƥ֤ȡ
     * Ū줿ǥեȤΥե졼֤
     * init() ƤӽФ˸Ƥ֤ȡޥɥ饤Ķѿ
     * Ť줿ե졼֤
     *
     * @else
     *
     * @brief Get configuration value.
     *
     * This operation returns default configuration statically defined,
     * when before calling init() function. When after calling init() function,
     * this operation returns initialized configuration value according to
     * command option, environment value and so on.
     *
     * @endif
     */
    Properties getConfig() const;

  protected:
    /*!
     * @if jp
     *
     * @brief ޥɰѡ
     *
     * -f file   : ե졼եꤹ롣<br>
     * -l module : ɤ⥸塼ꤹ롣<br>
     * -o options: ¾ץꤹ롣<br>
     * -d        : ǥեȤΥե졼Ȥ<br>
     *
     * @else
     *
     * @brief Parse command arguments
     *
     * -f file   : Specify a configuration file. <br>
     * -l module : Specify modules to be loaded at the beginning. <br>
     * -o options: Other options. <br>
     * -d        : Use default static configuration. <br>
     *
     * @endif
     */
    void parseArgs(int argc, char** argv);
    
    
    /*!
     * @if jp
     *
     * @brief Configuration file õ
     *
     * Configuration file ͥ
     *
     * ޥɥץĶѿǥեȥեǥե
     *
     * ǥեȶץ(-d): ǥեȥե뤬äƤ̵뤷
     *                               ǥեȤ
     *
     *
     * @else
     *
     * @brief Find configuration file
     *
     * @endif
     */
    bool findConfigFile();

    
    /*!
     * @if jp
     *
     * @brief ƥ򥻥åȤ
     *
     * ƥץѥƥ˥åȤ롣ꤵ륭ϰʲ̤ꡣ
     * manager.os.name    : OS̾
     * manager.os.release : OS꡼̾
     * maanger.os.version : OSС̾
     * manager.os.arch    : OSƥ
     * manager.os.hostname: ۥ̾
     * manager.pid        : ץID
     * 
     * @else
     * 
     * @brief Set system information
     * 
     * Get the following system info and set them to Manager's properties.
     * manager.os.name    : OS name
     * manager.os.release : OS release name
     * maanger.os.version : OS version
     * manager.os.arch    : OS architecture
     * manager.os.hostname: Hostname
     * manager.pid        : process ID
     *
     * @endif
     */
    void setSystemInformation(Properties& prop);


    /*!
     * @if jp
     * @brief ե뤬¸ߤ뤫ɤΤ
     * @else
     * @brief Check file existance
     * @endif
     */
    bool fileExist(const std::string& filename);

    std::string m_configFile;
  };
  
}; // namespace RTC  
  
  
#endif // ManagerConfig_h
  
