
// -*- C++ -*-
/*!
 * @file PortBase.h
 * @brief RTC's Port base class
 * @date $Date: 2007/09/20 11:26:08 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: PortBase.h,v 1.10.2.3 2007/09/20 11:26:08 n-ando Exp $
 *
 */

/*
 * $Log: PortBase.h,v $
 * Revision 1.10.2.3  2007/09/20 11:26:08  n-ando
 * A function getPortProfile() was added to get PortProfile locally.
 *
 * Revision 1.10.2.2  2007/09/19 07:42:01  n-ando
 * A trivial bug fix.
 *
 * Revision 1.10.2.1  2007/07/20 16:01:45  n-ando
 * Useless friend class declaration was deleted.
 *
 * Revision 1.10  2007/04/26 15:31:34  n-ando
 * The header include order was modified to define _REENTRANT before
 * including ace/config-lite.h in Linux systems.
 * In ace 5.4.7 or later, _REENTRANT flag should be defined explicitly.
 *
 * Revision 1.9  2007/04/13 15:53:03  n-ando
 * RTC::OK was changed to RTC::RTC_OK.
 *
 * Revision 1.8  2007/02/04 17:00:59  n-ando
 * Object reference variable was added.
 *
 * Revision 1.7  2007/01/04 00:43:35  n-ando
 * Now, notify_connect() and notify_disconnect() behavior can be customized
 * publishInterfaces(), subscribeInterfaces() and unsubscribeInterfaces().
 *
 * Revision 1.6  2006/12/02 18:50:22  n-ando
 * A trivial fix.
 *
 * Revision 1.5  2006/11/27 09:57:09  n-ando
 * addProvider() function was added for registration of provider.
 * addConsumer() function was added for registration of consumer.
 *
 * Revision 1.4  2006/11/06 01:46:47  n-ando
 * #include <assert.h> was added.
 *
 * Revision 1.3  2006/11/06 01:16:39  n-ando
 * Now PortBase doesn't depend on PortProfileHelper.
 * Class refference manual has been updated.
 *
 * Revision 1.2  2006/10/17 19:06:45  n-ando
 * connect(), disconnect() and disconnect_all() was implemented.
 *
 * Revision 1.1  2006/10/17 10:22:24  n-ando
 * The first commitment.
 *
 */

#ifndef PortBase_h
#define PortBase_h

#include <rtm/RTC.h>

#include <string>
#include <vector>
#include <ace/Guard_T.h>
#include <ace/Recursive_Thread_Mutex.h>
#include <rtm/idl/RTCSkel.h>
#include <rtm/CORBA_SeqUtil.h>
#include <rtm/NVUtil.h>

#include <iostream>
namespace RTC
{
  //  class ConsumerBase;

  /*!
   * @if jp
   * @class PortBase
   * @brief Port δ쥯饹
   *
   * RTC::Port δȤʤ륯饹
   * RTC::Port Ϥۤ UML Port γǰѾƤꡢۤƱΤΤȤߤʤ
   * ȤǤ롣RT ݡͥȤΥ󥻥ץȤˤƤϡ
   * Port ϥݡͥȤ°ݡͥȤ¾ΥݡͥȤߺ
   * ԤǤꡢ̾ĤΥ󥿡եȴϢդ롣
   * ݡͥȤ Port ̤ƳФ󥿡ե󶡤ޤ׵
   * 뤳ȤǤPortϤ³ô
   * <p>
   * Port ζݥ饹ϡ̾ RT ݡͥȥ󥹥Ʊ
   * 졢󶡡׵ᥤ󥿡եϿ塢RT ݡͥȤ
   * Ͽ졢饢ǽ Port ȤƵǽ뤳ȤꤷƤ롣
   * <p>
   * RTC::Port  CORBA 󥿡եȤưʲΥڥ졼󶡤롣
   *
   * - get_port_profile()
   * - get_connector_profiles()
   * - get_connector_profile()
   * - connect()
   * - notify_connect()
   * - disconnect()
   * - notify_disconnect()
   * - disconnect_all()
   *
   * Υ饹ǤϡΥڥ졼μ󶡤롣
   * <p>
   * Υڥ졼Τget_port_profile(), get_connector_profiles(),
   * get_connector_profile(), connect(), disconnect(), disconnect_all() ϡ
   * ֥饹ˤä˿񤤤ѹɬפʤᡢС饤
   * 뤳ȤϿ侩ʤ
   * <p>
   * notify_connect(), notify_disconnect() ˤĤƤϡ֥饹󶡡׵
   * 륤󥿡եμ˱ơ񤤤ѹɬפ
   * ⤷ʤľܥС饤ɤ뤳ȤϿ侩줺
   * Ҥ notify_connect(), notify_disconnect() ιˤƤҤ٤̤
   * δؿ˴Ϣ protected ؿ򥪡С饤ɤ뤳Ȥˤ
   * 񤤤ѹ뤳Ȥ侩롣
   *
   * @else
   *
   *
   * @endif
   */  
  class PortBase
    : public virtual POA_RTC::Port,
      public virtual PortableServer::RefCountServantBase
  {
  public:
    /*!
     * @if jp
     * @brief 󥹥ȥ饯
     *
     * PortBase Υ󥹥ȥ饯 Port ̾ name ˼Ԥ
     * ƱˡʬȤ CORBA Object ȤƳȤ PortProfile
     *  port_ref ˼ȤΥ֥ȥե󥹤Ǽ롣
     *
     * @param name Port ̾
     *
     * @else
     *
     * @brief Constructor
     *
     * The constructor of the ProtBase class is given the name of this Port
     * and initialized. At the same time, the PortBase activates itself
     * as CORBA object and stores its object reference to the PortProfile's 
     * port_ref member.
     *
     * @param name The name of Port 
     *
     * @endif
     */
    PortBase(const char* name = "");


    /*!
     * @if jp
     *
     * @brief ǥȥ饯
     *
     * @else
     *
     * @brief Destructor
     *
     * @endif
     */
    virtual ~PortBase();


    /*!
     * @if jp
     *
     * @brief [CORBA interface] PortProfile
     *
     * PortݻPortProfile֤
     * PortProfile ¤ΤϰʲΥСġ
     *
     * - name              [string ] Port ̾
     * - interfaces        [PortInterfaceProfileList ] Port ݻ
     *                     PortInterfaceProfile Υ
     * - port_ref          [Port Object ] Port ȤΥ֥ȥե
     * - connector_profile [ConnectorProfileList ] Port ݻ
     *                     ConnectorProfile Υ
     * - owner             [RTObject Object ]  Port ͭ
     *                     RTObjectΥե
     * - properties        [NVList ] ¾Υץѥƥ
     *
     * @return  Port  PortProfile
     *
     * @else
     *
     * @brief [CORBA interface] Get the PortProfile of the Port
     *
     * This operation returns the PortProfile of the Port.
     * PortProfile struct has the following members,
     *
     * - name              [string ] The name of the Port.
     * - interfaces        [PortInterfaceProfileList ] The sequence of 
     *                     PortInterfaceProfile owned by the Port
     * - port_ref          [Port Object] The object reference of the Port.
     * - connector_profile [ConnectorProfileList ] The sequence of 
     *                     ConnectorProfile owned by the Port.
     * - owner             [RTObject Object] The object reference of 
     *                     RTObject that is owner of the Port.
     * - properties        [NVList] The other properties.
     *
     * @return the PortProfile of the Port
     *
     * @endif
     */
    virtual PortProfile* get_port_profile();

    const PortProfile& getPortProfile() const;

    /*!
     * @if jp
     *
     * @brief [CORBA interface] ConnectorProfileList
     *
     * Portݻ ConnectorProfile  sequence ֤
     * ConnectorProfile  Port ֤³ץեݻ빽¤ΤǤꡢ
     * ³Port֤Ǿ򴹤ԤϢ뤹٤Ƥ Port Ʊͤ
     * ݻ롣
     * ConnectorProfile ϰʲΥСݻƤ롣
     *
     * - name         [string ] Υͥ̾
     * - connector_id [string ] ˡID
     * - ports        [Port sequnce] Υͥ˴Ϣ Port Υ֥
     *                ե󥹤Υ󥹡
     * - properties   [NVList ] ¾Υץѥƥ
     *
     * @return  Port  ConnectorProfile
     *
     * @else
     *
     * @brief [CORBA interface] Get the ConnectorProfileList of the Port
     *
     * This operation returns a list of the ConnectorProfiles of the Port.
     * ConnectorProfile includes the connection information that describes 
     * relation between (among) Ports, and Ports exchange the ConnectionProfile
     * on connection process and hold the same information in each Port.
     * ConnectionProfile has the following members,
     *
     * - name         [string] The name of the connection.
     * - connector_id [string] Unique identifier.
     * - ports        [Port sequnce] The sequence of Port's object reference
     *                that are related the connection.
     * - properties   [NVList] The other properties.
     *
     * @return the ConnectorProfileList of the Port
     *
     * @endif
     */
    virtual ConnectorProfileList* get_connector_profiles();


    /*!
     * @if jp
     *
     * @brief [CORBA interface] ConnectorProfile 
     *
     * connector_id ǻꤵ줿 ConnectorProfile ֤
     *
     * @param connector_id ConnectorProfile  ID
     * @return connector_id  ConnectorProfile
     *
     * @else
     *
     * @brief [CORBA interface] Get the ConnectorProfile
     *
     * This operation returns the ConnectorProfiles specified connector_id.
     *
     * @param connector_id ID of the ConnectorProfile
     * @return the ConnectorProfile identified by the connector_id
     *
     * @endif
     */
    virtual ConnectorProfile* get_connector_profile(const char* connector_id);


    /*!
     * @if jp
     *
     * @brief [CORBA interface] Port ³Ԥ
     *
     * Ϳ줿 ConnectoionProfile ˤäơPort֤³Ω롣
     * ץꥱץ¦ϡĤΥݡͥȤʣ
     * Port ³硢Ŭڤͤ򥻥åȤ ConnectorProfile 
     * connect() ΰȤͿƥ뤹뤳ȤˤꡢϢ Port 
     * ³Ω롣
     *
     * connect() Ϳ ConnectorProfile ΥСΤname, ports, 
     * (properties) СФƥǡ򥻥åȤʤФʤʤ
     *
     * @param connector_profile ConnectorProfile
     * @return ReturnCode_t ڥ졼Υ꥿󥳡
     *
     * @else
     *
     * @brief [CORBA interface] Connect the Port
     *
     * This operation establishes connection according to the given 
     * ConnectionProfile inforamtion. 
     * Application programs, which is going to establish the connection 
     * among Ports owned by RT-Components, have to set valid values to the 
     * ConnectorProfile and give it to the argument of connect() operation.
     * 
     * name, ports, (properties) members of ConnectorProfile should be set
     * valid values before giving to the argument of connect() operation.
     *
     * @param connector_profile The ConnectorProfile.
     * @return ReturnCode_t The return code of this operation.
     *
     * @endif
     */
    virtual ReturnCode_t connect(ConnectorProfile& connector_profile);


    /*!
     * @if jp
     *
     * @brief [CORBA interface] Port ³ΤԤ
     *
     * Υڥ졼ϡPort֤³ԤݤˡPort֤Ū
     * ƤФ륪ڥ졼Ǥ롣
     *
     * @param connector_profile ConnectorProfile
     * @return ReturnCode_t ڥ졼Υ꥿󥳡
     *
     * @else
     *
     * @brief [CORBA interface] Notify the Ports connection
     *
     * This operation is invoked between Ports internally when the connection
     * is established.
     *
     * @param connector_profile The ConnectorProfile.
     * @return ReturnCode_t The return code of this operation.
     *
     * @endif
     */
    virtual ReturnCode_t notify_connect(ConnectorProfile& connector_profile);


    /*!
     * @if jp
     *
     * @brief [CORBA interface] Port ³
     *
     * Υڥ졼³Ω³ФͿ connector_id 
     * бԥ Port Ȥ³롣
     *
     * @param connector_id ConnectorProfile  ID
     * @return ReturnCode_t ڥ졼Υ꥿󥳡
     *
     * @else
     *
     * @brief [CORBA interface] Connect the Port
     *
     * This operation destroys connection between this port and the peer port
     * according to given id that is given when the connection established.
     *
     * @param connector_id The ID of the ConnectorProfile.
     * @return ReturnCode_t The return code of this operation.
     *
     * @endif
     */
    virtual ReturnCode_t disconnect(const char* connector_id);


    /*!
     * @if jp
     *
     * @brief [CORBA interface] Port ³ΤԤ
     *
     * Υڥ졼ϡPort֤³ԤݤˡPort֤Ū
     * ƤФ륪ڥ졼Ǥ롣
     *
     * @param connector_id ConnectorProfile  ID
     * @return ReturnCode_t ڥ졼Υ꥿󥳡
     *
     * @else
     *
     * @brief [CORBA interface] Notify the Ports disconnection
     *
     * This operation is invoked between Ports internally when the connection
     * is destroied.
     *
     * @param connector_id The ID of the ConnectorProfile.
     * @return ReturnCode_t The return code of this operation.
     *
     * @endif
     */
    virtual ReturnCode_t notify_disconnect(const char* connector_id);


    /*!
     * @if jp
     *
     * @brief [CORBA interface] Port ³
     *
     * Υڥ졼Ϥ Port ˴ϢƤ³롣
     *
     * @return ReturnCode_t ڥ졼Υ꥿󥳡
     *
     * @else
     *
     * @brief [CORBA interface] Connect the Port
     *
     * This operation destroys all connection channels owned by the Port.
     *
     * @return ReturnCode_t The return code of this operation.
     *
     * @endif
     */
    virtual ReturnCode_t disconnect_all();


    //============================================================
    // Local operations
    //============================================================
    /*!
     * @if jp
     * @brief Port ̾ꤹ
     *
     * Port ̾ꤹ롣̾ Port ݻ PortProfile.name
     * ȿǤ롣
     *
     * @param name Port ̾
     *
     * @else
     * @brief Set the name of this Port
     *
     * This operation sets the name of this Port. The given Port's name is
     * applied to Port's PortProfile.name.
     *
     * @param name The name of this Port.
     *
     * @endif
     */
    void setName(const char* name);


    /*!
     * @if jp
     * @brief PortProfile
     *
     * Portݻ PortProfile  const Ȥ֤
     *
     * @return  Port  PortProfile
     *
     * @else
     * @brief Get the PortProfile of the Port
     *
     * This operation returns const reference of the PortProfile.
     *
     * @return the PortProfile of the Port
     *
     * @endif
     */
    const PortProfile& getProfile() const;
    

    /*!
     * @if jp
     *
     * @brief Port Υ֥ȻȤꤹ
     *
     * Υڥ졼 Port  PortProfile ˤ Port Ȥ
     * ֥ȻȤꤹ롣
     *
     * @param  Port Υ֥Ȼ
     *
     * @else
     *
     * @brief Set the object reference of this Port
     *
     * This operation sets the object reference itself
     * to the Port's PortProfile.
     *
     * @param The object reference of this Port.
     *
     * @endif
     */
    void setPortRef(Port_ptr port_ref);


    /*!
     * @if jp
     *
     * @brief Port Υ֥ȻȤ
     *
     * Υڥ졼 Port  PortProfile ݻƤ
     *  Port ȤΥ֥ȻȤ롣
     *
     * @return  Port Υ֥Ȼ
     *
     * @else
     *
     * @brief Get the object reference of this Port
     *
     * This operation returns the object reference
     * that is stored in the Port's PortProfile.
     *
     * @return The object reference of this Port.
     *
     * @endif
     */
    Port_ptr getPortRef();


    /*!
     * @if jp
     *
     * @brief Port  owner  RTObject ꤹ
     *
     * Υڥ졼 Port  PortProfile.owner ꤹ롣
     *
     * @param owner  Port ͭ RTObject λ
     *
     * @else
     *
     * @brief Set the owner RTObject of the Port
     *
     * This operation sets the owner RTObject of this Port.
     *
     * @param owner The owner RTObject's reference of this Port
     *
     * @endif
     */
    void setOwner(RTObject_ptr owner);


    //============================================================
    // protected operations
    //============================================================
  protected:
    /*! @if jp
     *
     * @brief Interface 
     *
     * Υڥ졼ϡnotify_connect() 󥹤λϤ˥
     * 貾۴ؿǤ롣
     * notify_connect() Ǥϡ
     *
     * - publishInterfaces()
     * - connectNext()
     * - subscribeInterfaces()
     * - updateConnectorProfile()
     *
     * ν protected ؿ뤵³Ԥ롣
     * <br>
     *  Port ǤϤΥڥ졼򥪡С饤ɤȤ
     * Ϳ줿 ConnectorProfile ˽Ԥѥ᡼Ŭ
     * ǤСRteurnCode_t Υ顼ɤ֤
     * ̾ publishInterafaces() ˤƤϡ Port °
     * 󥿡ե˴ؤ ConnectorProfile ФŬڤꤷ
     * ¾ Port ΤʤФʤʤ
     * <br>
     * ޤδؿ뤵ʳǤϡ¾ Port  Interface ˴ؤ
     * Ϥ٤ƴޤޤƤʤΤǡ¾ Port  Interface 
     *  subscribeInterfaces() ǹԤ٤Ǥ롣
     * <br>
     * Υڥ졼ϡ connector_id ФƤ³
     * ¸ connector_id ФƤϹŬڤ˹Ԥɬפ롣
     *
     * @param connector_profile ³˴ؤץե
     * @return ReturnCode_t Υ꥿󥳡
     *
     * @else
     *
     * @brief Publish interface information
     *
     * This operation is pure virutal method that would be called at the
     * beginning of the notify_connect() process sequence.
     * In the notify_connect(), the following methods would be called in order.
     *
     * - publishInterfaces()
     * - connectNext()
     * - subscribeInterfaces()
     * - updateConnectorProfile() 
     *
     * In the concrete Port, this method should be overridden. This method
     * processes the given ConnectorProfile argument and if the given parameter
     * is invalid, it would return error code of ReturnCode_t.
     * Usually, publishInterfaces() method should set interfaces information
     * owned by this Port, and publish it to the other Ports.
     * <br>
     * When this method is called, other Ports' interfaces information may not
     * be completed. Therefore, the process to obtain other Port's interfaces
     * information should be done in the subscribeInterfaces() method.
     * <br>
     * This operation should create the new connection for the new
     * connector_id, and should update the connection for the existing
     * connection_id.
     *
     * @param connector_profile The connection profile information
     * @return The return code of ReturnCode_t type.
     *
     * @endif
     */
    virtual ReturnCode_t
    publishInterfaces(ConnectorProfile& connector_profile) = 0;
    

    /*!
     * @if jp
     *
     * @brief  Port Ф notify_connect() 򥳡뤹
     *
     * ConnectorProfile  port_ref ˳ǼƤ Port Υ֥
     * ե󥹤Υ󥹤椫顢Ȥ Port μ Port Ф
     * notify_connect() 򥳡뤹롣
     *
     * @param connector_profile ³˴ؤץե
     * @return ReturnCode_t Υ꥿󥳡
     *
     * @else
     *
     * @brief Call notify_connect() of the next Port
     *
     * This operation calls the notify_connect() of the next Port's 
     * that stored in ConnectorProfile's port_ref sequence.
     *
     * @param connector_profile The connection profile information
     * @return The return code of ReturnCode_t type.
     *
     * @endif
     */
    virtual ReturnCode_t connectNext(ConnectorProfile& connector_profile);


    /*!
     * @if jp
     *
     * @brief  Port Ф notify_disconnect() 򥳡뤹
     *
     * ConnectorProfile  port_ref ˳ǼƤ Port Υ֥
     * ե󥹤Υ󥹤椫顢Ȥ Port μ Port Ф
     * notify_disconnect() 򥳡뤹롣
     *
     * @param connector_profile ³˴ؤץե
     * @return ReturnCode_t Υ꥿󥳡
     *
     * @else
     *
     * @brief Call notify_disconnect() of the next Port
     *
     * This operation calls the notify_disconnect() of the next Port's 
     * that stored in ConnectorProfile's port_ref sequence.
     *
     * @param connector_profile The connection profile information
     * @return The return code of ReturnCode_t type.
     *
     * @endif
     */
    virtual ReturnCode_t disconnectNext(ConnectorProfile& connector_profile);


    /*! @if jp
     *
     * @brief Interface 
     *
     * Υڥ졼ϡnotify_connect() 󥹤֤˥
     * 貾۴ؿǤ롣
     * notify_connect() Ǥϡ
     *
     * - publishInterfaces()
     * - connectNext()
     * - subscribeInterfaces()
     * - updateConnectorProfile()
     *
     * ν protected ؿ뤵³Ԥ롣
     * <br>
     *  Port ǤϤΥڥ졼򥪡С饤ɤȤ
     * Ϳ줿 ConnectorProfile ˽Ԥѥ᡼Ŭ
     * ǤСRteurnCode_t Υ顼ɤ֤
     *  ConnectorProfile ˤ¾ Port  Interface ˴ؤ
     * ƴޤޤƤ롣
     * ̾ subscribeInterafaces() ˤƤϡ Port Ѥ
     * Interface ˴ؤ׵¦Υ󥿡եФ
     * ꤷʤФʤʤ
     * <br>
     * Υڥ졼ϡ connector_id ФƤ³
     * ¸ connector_id ФƤϹŬڤ˹Ԥɬפ롣
     *
     * @param connector_profile ³˴ؤץե
     * @return ReturnCode_t Υ꥿󥳡
     *
     * @else
     *
     * @brief Publish interface information
     *
     * This operation is pure virutal method that would be called at the
     * mid-flow of the notify_connect() process sequence.
     * In the notify_connect(), the following methods would be called in order.
     *
     * - publishInterfaces()
     * - connectNext()
     * - subscribeInterfaces()
     * - updateConnectorProfile()
     *
     * In the concrete Port, this method should be overridden. This method
     * processes the given ConnectorProfile argument and if the given parameter
     * is invalid, it would return error code of ReturnCode_t.
     * The given argument ConnectorProfile includes all the interfaces
     * information in it.
     * Usually, subscribeInterafaces() method obtains information of interfaces
     * from ConnectorProfile, and should set it to the interfaces that require
     * them.
     * <br>
     * This operation should create the new connection for the new
     * connector_id, and should update the connection for the existing
     * connection_id.
     *
     * @param connector_profile The connection profile information
     * @return The return code of ReturnCode_t type.
     *
     * @endif
     */
    virtual ReturnCode_t
    subscribeInterfaces(const ConnectorProfile& connector_profile) = 0;
    
    
    /*! @if jp
     *
     * @brief Interface ³
     *
     * Υڥ졼ϡnotify_disconnect() 󥹤ν˥
     * 貾۴ؿǤ롣
     * notify_disconnect() Ǥϡ
     * - disconnectNext()
     * - unsubscribeInterfaces()
     * - eraseConnectorProfile()
     * ν protected ؿ뤵³Ԥ롣
     * <br>
     *  Port ǤϤΥڥ졼򥪡С饤ɤȤ
     * Ϳ줿 ConnectorProfile ˽³Ԥ
     *
     * @param connector_profile ³˴ؤץե
     *
     * @else
     *
     * @brief Disconnect interface connection
     *
     * This operation is pure virutal method that would be called at the
     * end of the notify_disconnect() process sequence.
     * In the notify_disconnect(), the following methods would be called.
     * - disconnectNext()
     * - unsubscribeInterfaces()
     * - eraseConnectorProfile() 
     *  <br>
     * In the concrete Port, this method should be overridden. This method
     * processes the given ConnectorProfile argument and disconnect interface
     * connection.
     *
     * @param connector_profile The connection profile information
     *
     * @endif
     */
    virtual void
    unsubscribeInterfaces(const ConnectorProfile& connector_profile) = 0;


    //============================================================
    // protected utility functions
    //============================================================
    /*!
     * @if jp
     *
     * @brief ConnectorProfile  connector_id եɤɤȽ
     *
     * @return Ϳ줿 ConnectorProfile  connector_id ǤС
     *         trueǤʤ false ֤
     *
     * @else
     *
     * @brief Whether connector_id of ConnectorProfile is empty
     *
     * @return If the given ConnectorProfile's connector_id is empty string,
     *         it returns true.
     *
     * @endif
     */
    bool isEmptyId(const ConnectorProfile& connector_profile) const;


    /*!
     * @if jp
     *
     * @brief UUID
     *
     * Υڥ졼 UUID 롣
     *
     * @return uuid
     *
     * @else
     *
     * @brief Get the UUID
     *
     * This operation generates UUID.
     *
     * @return uuid
     *
     * @endif
     */
    const std::string getUUID() const;


    /*!
     * @if jp
     *
     * @brief UUID ConnectorProfile ˥åȤ
     *
     * Υڥ졼 UUID ConnectorProfile ˥åȤ롣
     *
     * @param connector_profile connector_id 򥻥åȤ ConnectorProfile
     *
     * @else
     *
     * @brief Create and set the UUID to the ConnectorProfile
     *
     * This operation generates and set UUID to the ConnectorProfile.
     *
     * @param connector_profile ConnectorProfile to be set connector_id
     *
     * @endif
     */
    void setUUID(ConnectorProfile& connector_profile) const;


    /*!
     * @if jp
     *
     * @brief id ¸ ConnectorProfile ΤΤɤȽꤹ
     *
     * Υڥ졼Ϳ줿 ID ¸ ConnectorProfile Υꥹ
     * ¸ߤ뤫ɤȽꤹ롣
     *
     * @param id Ƚꤹ connector_id
     *
     * @else
     *
     * @brief Whether the given id exists in stored ConnectorProfiles
     *
     * This operation returns boolean whether the given id exists in 
     * the Port's ConnectorProfiles.
     *
     * @param id connector_id to be find in Port's ConnectorProfiles
     *
     * @endif
     */
    bool isExistingConnId(const char* id);


    /*!
     * @if jp
     *
     * @brief id  ConnectorProfile õ
     *
     * Υڥ졼Ϳ줿 ID  ConnectorProfile  Port 
     *  ConnectorProfile Υꥹ椫õ
     * ⤷Ʊ id  ConnectorProfile ʤС ConnectorProfile
     * ֤롣
     *
     * @param id  connector_id
     * @return connector_id  ConnectorProfile
     *
     * @else
     *
     * @brief Find ConnectorProfile with id
     *
     * This operation returns ConnectorProfile with the given id from Port's
     * ConnectorProfiles' list.
     * If the ConnectorProfile with connector id that is identical with the
     * given id does not exist, empty ConnectorProfile is returned.
     *
     * @param id the connector_id to be searched in Port's ConnectorProfiles
     * @return CoonectorProfile with connector_id
     *
     * @endif
     */
    ConnectorProfile findConnProfile(const char* id);
    

    /*!
     * @if jp
     *
     * @brief id  ConnectorProfile õ
     *
     * Υڥ졼Ϳ줿 ID  ConnectorProfile  Port 
     *  ConnectorProfile Υꥹ椫õǥå֤
     * ⤷Ʊ id  ConnectorProfile ʤС-1 ֤
     *
     * @param id  connector_id
     * @return Port  ConnectorProfile ꥹȤΥǥå
     *
     * @else
     *
     * @brief Find ConnectorProfile with id
     *
     * This operation returns ConnectorProfile with the given id from Port's
     * ConnectorProfiles' list.
     * If the ConnectorProfile with connector id that is identical with the
     * given id does not exist, empty ConnectorProfile is returned.
     *
     * @param id the connector_id to be searched in Port's ConnectorProfiles
     * @return The index of ConnectorProfile of the Port
     *
     * @endif
     */
    CORBA::Long findConnProfileIndex(const char* id);


    /*!
     * @if jp
     *
     * @brief ConnectorProfile ɲä⤷Ϲ
     *
     * Υڥ졼Ϳ줿Ϳ줿 ConnectorProfile 
     * Port ɲä⤷Ϲ¸롣
     * Ϳ줿 ConnectorProfile  connector_id Ʊ ID 
     * ConnectorProfile ꥹȤˤʤСꥹȤɲä
     * Ʊ ID ¸ߤ ConnectorProfile ¸롣
     *
     * @param coonector_profile ɲä⤷Ϲ ConnectorProfile
     *
     * @else
     *
     * @brief Append or update the ConnectorProfile list
     *
     * This operation appends or updates ConnectorProfile of the Port
     * by the given ConnectorProfile.
     * If the connector_id of the given ConnectorProfile does not exist
     * in the Port's ConnectorProfile list, the given ConnectorProfile would be
     * append to the list. If the same id exists, the list would be updated.
     *
     * @param connector_profile the ConnectorProfile to be appended or updated
     *
     * @endif
     */
    void updateConnectorProfile(const ConnectorProfile& connector_profile);


    /*!
     * @if jp
     *
     * @brief ConnectorProfile 
     *
     * Υڥ졼 Port  PortProfile ݻƤ
     * ConnectorProfileList ΤͿ줿 id  ConnectorProfile
     * 롣
     *
     * @param id  ConnectorProfile  id
     *
     * @else
     *
     * @brief Delete the ConnectorProfile
     *
     * This operation deletes a ConnectorProfile specified by id from
     * ConnectorProfileList owned by PortProfile of this Port.
     *
     * @param id The id of the ConnectorProfile to be deleted.
     *
     * @endif
     */
    bool eraseConnectorProfile(const char* id);


    /*!
     * @if jp
     *
     * @brief PortInterfaceProfile  󥿡եϿ
     *
     * Υڥ졼 Port  PortProfile ΡPortInterfaceProfile
     * ˥󥿡եξɲä롣
     * ξϡget_port_profile() ä PortProfile Τ
     * PortInterfaceProfile ͤѹΤߤǤꡢºݤ˥󥿡ե
     * 󶡤׵ᤷꤹˤϡ֥饹ǡpublishInterface(),
     * subscribeInterface() δؿŬڤ˥С饤ɤ󥿡ե
     * 󶡡׵ԤʤФʤʤ
     *
     * 󥿡ե(Υ󥹥)̾ Port ǰդǤʤФʤʤ
     * Ʊ̾Υ󥿡եǤϿƤ硢δؿ false 
     * ֤
     *
     * @param name 󥿡եΥ󥹥󥹤̾
     * @param type_name 󥿡եη̾
     * @param pol 󥿡ե° (RTC::PROVIDED ⤷ RTC:REQUIRED)
     * @return Ʊ̾Υ󥿡եϿƤ false ֤
     *
     * @else
     *
     * @brief Append an interface to the PortInterfaceProfile
     *
     * This operation appends interface information to the PortInterfaceProfile
     * that is owned by the Port.
     * The given interfaces information only updates PortInterfaceProfile of
     * PortProfile that is obtained through get_port_profile().
     * In order to provide and require interfaces, proper functions (for
     * example publishInterface(), subscribeInterface() and so on) should be
     * overridden in subclasses, and these functions provide concrete interface
     * connection and disconnection functionality.
     *
     * The interface (instance) name have to be unique in the Port.
     * If the given interface name is identical with stored interface name,
     * this function returns false.
     *
     * @param name The instance name of the interface.
     * @param type_name The type name of the interface.
     * @param pol The interface's polarity (RTC::PROVIDED or RTC:REQUIRED)
     * @return false would be returned if the same name is already registered.
     *
     * @endif
     */
    bool appendInterface(const char* name, const char* type_name,
			 PortInterfacePolarity pol);


    /*!
     * @if jp
     *
     * @brief PortInterfaceProfile 饤󥿡եϿ
     *
     * Υڥ졼 Port  PortProfile ΡPortInterfaceProfile
     * 饤󥿡եξ롣
     *
     * @param name 󥿡եΥ󥹥󥹤̾
     * @param pol 󥿡ե° (RTC::PROVIDED ⤷ RTC:REQUIRED)
     * @return 󥿡եϿƤʤ false ֤
     *
     * @else
     *
     * @brief Delete an interface from the PortInterfaceProfile
     *
     * This operation deletes interface information from the
     * PortInterfaceProfile that is owned by the Port.
     *
     * @param name The instance name of the interface.
     * @param pol The interface's polarity (RTC::PROVIDED or RTC:REQUIRED)
     * @return false would be returned if the given name is not registered.
     *
     * @endif
     */
    bool deleteInterface(const char* name, PortInterfacePolarity pol);
    

    /*!
     * @if jp
     *
     * @brief PortProfile  properties  NameValue ͤɲä
     *
     * @param ValueType properties  value ɲäͤη
     * @param key properties  name
     * @param value properties  value
     *
     * @else
     *
     * @brief Add NameValue data to PortProfile's properties
     *
     * @param ValueType The type of value
     * @param key The name of properties
     * @param value The value of properties
     *
     * @endif
     */
    template <class ValueType>
    void addProperty(const char* key, ValueType value)
    {
      CORBA_SeqUtil::push_back(m_profile.properties,
			       NVUtil::newNV(key, value));
    }


  protected:
    /*!
     * @if jp
     * @brief Port  PortProfile
     * @else
     * @brief PortProfile of the Port
     * @endif
     */
    PortProfile m_profile;
    RTC::Port_var m_objref;
    mutable ACE_Recursive_Thread_Mutex m_profile_mutex;
    typedef ACE_Guard<ACE_Recursive_Thread_Mutex> Guard;
    
    //============================================================
    // Functor
    //============================================================
    /*!
     * @if jp
     * @brief instance_name  PortInterfaceProfile õ Functor
     * @else
     * @brief A functor to find a PortInterfaceProfile named instance_name
     * @endif
     */
    struct if_name
    {
      if_name(const char* name) : m_name(name) {};
      bool operator()(const PortInterfaceProfile& prof)
      {
	return m_name == std::string(prof.instance_name);
      }
      std::string m_name;
    };
    
    
    /*!
     * @if jp
     * @brief id  ConnectorProfile õ Functor
     * @else
     * @brief A functor to find a ConnectorProfile named id
     * @endif
     */
    struct find_conn_id
    {
      find_conn_id(const char* id) : m_id(id) {};
      bool operator()(const ConnectorProfile& cprof)
      {
	return m_id == std::string(cprof.connector_id);
      }
      std::string m_id;
    };


    /*!
     * @if jp
     * @brief 󥹥ȥ饯 port_ref Ʊ֥ȻȤõ Functor
     * @else
     * @brief A functor to find the object reference that is identical port_ref
     * @endif
     */
    struct find_port_ref
    {
      find_port_ref(Port_ptr port_ref) : m_port(port_ref) {};
      bool operator()(Port_ptr port_ref)
      {
	return m_port->_is_equivalent(port_ref);
      }
      Port_ptr m_port;
    };
	
    
    /*!
     * @if jp
     * @brief Port ³Ԥ Functor
     * @else
     * @brief A functor to connect Ports
     * @endif
     */
    struct connect_func
    {
      Port_var port_ref;
      ConnectorProfile connector_profile;
      ReturnCode_t return_code;
      
      connect_func() {};
      connect_func(Port_ptr p, ConnectorProfile& prof)
	: port_ref(p), connector_profile(prof), return_code(RTC::RTC_OK) {};
      void operator()(Port_ptr p)
      {
	if (!port_ref->_is_equivalent(p))
	  {
	    ReturnCode_t retval;
	    retval = p->notify_connect(connector_profile);
	    if (retval != RTC::RTC_OK)
	      {
		return_code = retval;
	      }
	  }
      }
    };
    
    
    /*!
     * @if jp
     * @brief Port ³Ԥ Functor
     * @else
     * @brief A functor to disconnect Ports
     * @endif
     */
    struct disconnect_func
    {
      Port_var port_ref;
      ConnectorProfile connector_profile;
      ReturnCode_t return_code;
      
      disconnect_func() : return_code(RTC::RTC_OK) {};
      disconnect_func(Port_ptr p, ConnectorProfile& prof)
	: port_ref(p), connector_profile(prof), return_code(RTC::RTC_OK) {};
      void operator()(Port_ptr p)
      {
	if (!port_ref->_is_equivalent(p))
	  {
	    ReturnCode_t retval;
	    retval = p->disconnect(connector_profile.connector_id);
	    if (retval != RTC::RTC_OK)
	      {
		return_code = retval;
	      }
	  }
      }
    };
    
    
    /*!
     * @if jp
     * @brief Port ³Ԥ Functor
     * @else
     * @brief A functor to disconnect all Ports
     * @endif
     */
    struct disconnect_all_func
    {
      ReturnCode_t return_code;
      PortBase* port;
      
      disconnect_all_func() {};
      disconnect_all_func(PortBase* p) 
	: return_code(RTC::RTC_OK), port(p) {};
      void operator()(ConnectorProfile& p)
      {
	ReturnCode_t retval;
	retval = port->disconnect(p.connector_id);
	if (retval != RTC::RTC_OK)
	  {
	    return_code = retval;
	  }
      }
    };
    

    /*!
     * @if jp
     * @brief name  polarity  interface õ Functor
     * @else
     * @brief A functor to find interface from name and polarity
     * @endif
     */
    struct find_interface
    {
      find_interface(const char* name, PortInterfacePolarity pol)
	: m_name(name), m_pol(pol)
      {}

      bool operator()(const PortInterfaceProfile& prof)
      {
	std::string name(CORBA::string_dup(prof.instance_name));
	return ((m_name == name) && (m_pol == prof.polarity));
      }
      std::string m_name;
      PortInterfacePolarity m_pol;
    };
  };
};
#endif // PortBase_h
