// -*- C++ -*-
/*!
 * @file RingBuffer.h
 * @brief Defautl Buffer class
 * @date $Date: 2007/07/20 16:07:58 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: RingBuffer.h,v 1.5.2.1 2007/07/20 16:07:58 n-ando Exp $
 *
 */

/*
 * $Log: RingBuffer.h,v $
 * Revision 1.5.2.1  2007/07/20 16:07:58  n-ando
 * Shimizu's patch to modify vector access violation.
 *
 * Revision 1.5  2007/04/26 15:32:56  n-ando
 * The header include order was modified to define _REENTRANT before
 * including ace/config-lite.h in Linux systems.
 * In ace 5.4.7 or later, _REENTRANT flag should be defined explicitly.
 *
 * Revision 1.4  2007/01/06 18:02:49  n-ando
 * The RingBuffer has been completely re-implemented.
 *
 *
 */

#ifndef RingBuffer_h
#define RingBuffer_h

#include <rtm/RTC.h>

#include <vector>
#include <algorithm>
//#include <ace/Synch.h>
#include <rtm/BufferBase.h>

namespace RTC
{
  template <class DataType>
  class RingBuffer
    : public BufferBase<DataType>
  {
  public:
    RingBuffer(long int length)
      : m_length(length < 2 ? 2 : length),
	m_oldPtr(0),
	m_newPtr(length < 2 ? 1 : length - 1)
    {
      m_buffer.resize(m_length);
    }

    /*!
     * @if jp
     *
     * @brief ۥǥȥ饯
     * 
     * @else
     *
     * @brief virtual destractor
     *
     * @endif
     */
    virtual ~RingBuffer(){};
    

    void init(DataType& data)
    {
      for (long int i = 0; i < m_length; ++i)
	{
	  put(data);
	}
    }

    /*!
     * @if jp
     *
     * @brief ХåեĹ
     * 
     * @else
     *
     * @brief Get the buffer length
     *
     * @endif
     */
    virtual long int length() const
    {
      return m_length;
    }

    /*!
     * @if jp
     *
     * @brief Хåե˥ǡ񤭹
     * 
     * @else
     *
     * @brief Write data into the buffer
     *
     * @endif
     */
    virtual int write(const DataType& value)
    {
      int status = put(value);
      return status;
    }

    /*!
     * @if jp
     *
     * @brief Хåե˥ǡ񤭹
     * 
     * @else
     *
     * @brief Write data into the buffer
     *
     * @endif
     */
    virtual int read(DataType& value)
    {
     int status = get(value);
      return status;
    }

    /*!
     * @if jp
     *
     * @brief ХåեfullǤ
     * 
     * @else
     *
     * @brief True if the buffer is full, else false.
     *
     * @endif
     */
    virtual bool isFull() const
    {
      return false;
    }
    /*!
     * @if jp
     *
     * @brief ХåեemptyǤ
     * 
     * @else
     *
     * @brief True if the buffer is empty, else false.
     *
     * @endif
     */
    virtual bool isEmpty() const
    {
      return !(this->isNew());
    }

    bool isNew() const
    {
      return m_buffer[m_newPtr].isNew();
    }

  protected:
    /*!
     * @if jp
     *
     * @brief Хåե˥ǡ񤭹
     * 
     * @else
     *
     * @brief Write data into the buffer
     *
     * @endif
     */
    virtual int put(const DataType& data)
    {
     int status = m_buffer[m_oldPtr].write(data);
      //      ACE_Guard<ACE_Thread_Mutex> guard(m_Mutex);
      m_newPtr = m_oldPtr;
      m_oldPtr = (++m_oldPtr) % m_length;
      return status;
    }

    /*!
     * @if jp
     *
     * @brief Хåեǡ
     * 
     * @else
     *
     * @brief Get data from the buffer
     *
     * @endif
     */
    virtual int get(DataType& data)
    {
      int status = m_buffer[m_newPtr].read();
      return status;
    }

    /*!
     * @if jp
     *
     * @brief ˽񤭹ХåեλȤ
     * 
     * @else
     *
     * @brief Get the buffer's reference to be written the next
     *
     * @endif
     */
    virtual DataType& getRef()
    {
      return m_buffer[m_newPtr].data;
    }

  private:
    long int m_length;
    long int m_oldPtr;
    long int m_newPtr;

    /*!
     * @if jp
     * @brief Хåե
     * @else
     * @brief Buffer sequence
     * @endif
     */
    template <class D>
    class Data
    {
    public:
      Data() : data(), is_new(false){;}
      inline Data& operator=(const D& other)
      {
        this->data = other;
        this->is_new = true;
        return *this;
      }
      inline void write(const D& other)
      {
        this->is_new = true;
        this->data = other;
      }
      inline D& read()
      {
        this->is_new = false;
        return this->data;
      }
      inline bool isNew() const
      {
        return is_new;
      }
      D    data;
      bool is_new;
    };

    std::vector<Data<DataType> > m_buffer;

  };
}; // namespace RTC

#endif // RingBuffer_h
