// -*- C++ -*-
/*!
 * @file SdoConfiguration.h
 * @brief RT component base class
 * @date $Date: 2007/09/19 03:01:42 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: SdoConfiguration.h,v 1.8.2.1 2007/09/19 03:01:42 n-ando Exp $
 *
 */

/*
 * $Log: SdoConfiguration.h,v $
 * Revision 1.8.2.1  2007/09/19 03:01:42  n-ando
 * Inconsistency between idl and impl. in get_configuration_set() was modified.
 *
 * Revision 1.8  2007/04/26 15:33:21  n-ando
 * The header include order was modified to define _REENTRANT before
 * including ace/config-lite.h in Linux systems.
 * In ace 5.4.7 or later, _REENTRANT flag should be defined explicitly.
 *
 * Revision 1.7  2007/04/23 04:58:21  n-ando
 * SDO Configuration was modified to use Configuration admin class.
 *
 * Revision 1.6  2007/01/24 16:03:58  n-ando
 * The ctor. was changed.
 *
 * Revision 1.5  2007/01/21 13:05:13  n-ando
 * A trivial fix.
 *
 * Revision 1.4  2006/11/08 20:00:21  n-ando
 * ConfigurationSet related interfaces are fixed.
 *
 * Revision 1.3  2006/10/30 08:05:45  n-ando
 * CORBA sequence operations were replaced by CORBA_SeqUtil functions.
 *
 * Revision 1.2  2006/10/17 10:13:15  n-ando
 * Small fixes.
 *
 * Revision 1.1  2006/09/11 18:14:01  n-ando
 * The first commit.
 *
 *
 */

#ifndef SdoConfiguration_h
#define SdoConfiguration_h

// CORBA header include
#include <rtm/RTC.h>

#include <ace/Guard_T.h>
#include <ace/Thread_Mutex.h>

// local includes
#include <rtm/idl/SDOPackageSkel.h>
#include <rtm/ConfigAdmin.h>
#include <string>

// SdoConfiguration with SeqEx 159120
// SdoConfiguration with SeqUtil 114504 114224

namespace SDOPackage
{
  /*!
   * @if jp
   *
   * @class Configuration_impl
   * @brief SDO Configuration 饹
   *
   * Configuration interface  Resource Data Model 줿ǡ
   * ɲáԤΥ󥿡եǤ롣
   * DeviceProfile, ServiceProfile, ConfigurationProfile  Organization
   * ѹԤΥڥ졼Ƥ롣SDO λͤǤϥ
   * ӥƥ˴ؤܺ٤ˤĤƤϵꤷƤʤ
   * 
   * ʣ (Configuration) ݻ뤳Ȥˤꡢưפ᤯
   * ȿǤ뤳ȤǤ롣줿ʣ ConfigurationSets
   *  configuration profile Ȥݻ뤳ȤǤ롣ҤȤĤ
   * ConfigurationSet ˴Ϣդ줿ץѥƥͤΥꥹȤ
   * ˡIDܺ٤ȤȤ˻äƤ롣ˤꡢܤξܺ٤򵭽Ҥ
   * ̤뤳ȤǤ롣Configuration interface Υڥ졼Ϥ
   * ConfiguratioinSets δٱ礹롣
   *
   *
   * - ConfigurationSet: id, description, NVList 鹽1åȤ
   * - ConfigurationSetList: ConfigurationSet Υꥹ
   * - Parameter: name, type, allowed_values 鹽ѥ᡼
   * - ActiveConfigurationSet: ͭʥե졼1åȡ
   *
   * ʲSDOͤƤʤ⤷ϲ᤬狼ʤȼ
   *
   * ʲδؿ ParameterList ФƽԤ
   * - get_configuration_parameters()
   *
   * ʲδؿϥƥ֤ConfigurationSetФԤ
   * - get_configuration_parameter_values()
   * - get_configuration_parameter_value()
   * - set_configuration_parameter()
   *
   * ʲδؿConfigurationSetListФƽԤ
   * - get_configuration_sets()
   * - get_configuration_set()
   * - set_configuration_set_values()
   * - get_active_configuration_set()
   * - add_configuration_set()
   * - remove_configuration_set()
   * - activate_configuration_set()
   *
   * @else
   *
   * @class Configuration_impl
   * @brief Configuration implementation class
   *
   * Configuration interface provides operations to add or remove data
   * specified in resource data model. These operations provide functions to
   * change DeviceProfile, ServiceProfile, ConfigurationProfile, and
   * Organization. This specification does not address access control or
   * security aspects. Access to operations that modifies or removes profiles
   * should be controlled depending upon the application.
   *
   * Different configurations can be stored for simple and quick activation.
   * Different predefined configurations are stored as different
   * ConfigurationSets or configuration profile. A ConfigurationSet stores the
   * value of all properties assigned for the particular configuration along
   * with its unique id and description to identify and describe the
   * configuration respectively. Operations in the configuration interface
   * help manage these ConfigurationSets.
   *
   * @endif
   */
  class Configuration_impl
    : public virtual POA_SDOPackage::Configuration,
      public virtual PortableServer::RefCountServantBase
  {
  public:
    Configuration_impl(RTC::ConfigAdmin& configAdmin);
    virtual ~Configuration_impl();

    //============================================================
    //
    // <<< CORBA interfaces >>>
    //
    //============================================================
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] SDO  DeviceProfile 򥻥åȤ
     *
     * Υڥ졼 SDO  DeviceProfile 򥻥åȤ롣SDO 
     * DeviceProfile ݻƤϿ DeviceProfile 
     * DeviceProfile 򤹤ǤݻƤϴ¸ΤΤ֤롣
     *
     * @param dProfile SDO ˴Ϣդ DeviceProfile
     * @return ڥ졼ɤ֤
     * @exception SDONotExists åȤSDO¸ߤʤ
     * @exception NotAvailable SDO¸ߤ뤬ʤ
     * @exception InvalidParameter  "dProfile"  null Ǥ롣
     * @exception InternalError Ū顼ȯ
     * 
     * @else
     *
     * @brief [CORBA interface] Set DeviceProfile of SDO
     *
     * This operation sets the DeviceProfile of an SDO. If the SDO does not
     * have DeviceProfile, the operation will create a new DeviceProfile,
     * otherwise it will replace the existing DeviceProfile.
     *
     * @param dProfile The device profile that is to be assigned to this SDO.
     * @return If the operation was successfully completed.
     * @exception SDONotExists The target SDO does not exist.
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InvalidParameter The argument "dProfile" is null.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Boolean set_device_profile(const DeviceProfile& dProfile)
      throw (InvalidParameter, NotAvailable, InternalError);

    /*!
     * @if jp
     * 
     * @brief [CORBA interface] SDO  ServiceProfile Υå
     *
     * Υڥ졼Ϥ Configuration interface ͭо SDO 
     * ServiceProfile ꤹ롣⤷ ServiceProfile  id Ǥ
     *  ID 줽 ServiceProfile Ǽ롣⤷ id 
     * ʤСSDO Ʊ id  ServiceProfile 򸡺롣
     * Ʊ id ¸ߤʤФ ServiceProfile ɲäid ¸ߤ
     * 񤭤򤹤롣
     *
     * @param sProfile ɲä ServiceProfile
     * @return ڥ졼ɤ֤
     * @exception SDONotExists åȤSDO¸ߤʤ
     * @exception InvalidParameter  "sProfile"  nullǤ롣
     * @exception NotAvailable SDO¸ߤ뤬ʤ
     * @exception InternalError Ū顼ȯ
     * @else
     *
     * @brief [CORBA interface] Set SDO's ServiceProfile
     *
     * This operation adds ServiceProfile to the target SDO that navigates this
     * Configuration interface. If the id in argument ServiceProfile is null,
     * new id is created and the ServiceProfile is stored. If the id is not
     * null, the target SDO searches for ServiceProfile in it with the same id.
     * It adds the ServiceProfile if not exist, or overwrites if exist.
     *
     * @param sProfile ServiceProfile to be added.
     * @return If the operation was successfully completed.
     * @exception SDONotExists The target SDO does not exist.
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InvalidParameter The argument "sProfile" is null.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Boolean set_service_profile(const ServiceProfile& sProfile)
      throw (InvalidParameter, NotAvailable, InternalError);

    /*!
     * @if jp
     * 
     * @brief [CORBA interface] Organization ɲ
     *
     * Υڥ졼 Organization object Υե󥹤ɲä롣
     *
     * @param org ɲä Organization
     * @return ڥ졼ɤ֤
     * @exception SDONotExists åȤSDO¸ߤʤ
     * @exception NotAvailable SDO¸ߤ뤬ʤ
     * @exception InvalidParameter  "organization"  null Ǥ롣
     * @exception InternalError Ū顼ȯ
     * @else
     *
     * @brief [CORBA interface] Add Organization
     *
     * This operation adds reference of an Organization object.
     *
     * @param org Organization to be added.
     * @return If the operation was successfully completed.
     * @exception SDONotExists The target SDO does not exist.
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InvalidParameter The argument organization is null.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Boolean add_organization(Organization_ptr org)
      throw (InvalidParameter, NotAvailable, InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [CORBA interface] ServiceProfile κ
     *
     * Υڥ졼Ϥ Configuration interface  SDO 
     * Service  ServiceProfile 롣 ServiceProfile
     * ϰˤꤵ롣
     *
     * @param id  ServcieProfile  serviceID
     * @return ڥ졼ɤ֤
     * @exception SDONotExists åȤSDO¸ߤʤ
     * @exception InvalidParameter  "id"  null Ǥ롣⤷ "id" 
     *                             Ϣդ줿 ServiceProfile ¸ߤʤ
     * @exception NotAvailable SDO¸ߤ뤬ʤ
     * @exception InternalError Ū顼ȯ
     * @else
     *
     * @brief [CORBA interface] Remove ServiceProfile
     *
     * This operation removes ServiceProfile object to the SDO that has this
     * Configuration interface. The ServiceProfile object to be removed is
     * specified by argument.
     *
     * @param id serviceID of a ServiceProfile to be removed.
     * @return If the operation was successfully completed.
     * @exception SDONotExists The target SDO does not exist.
     * @exception InvalidParameter The argument "sProfile" is null, or if the
     *          object that is specified by argument "sProfile" does not exist.
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Boolean remove_service_profile(const char* id)
      throw (InvalidParameter, NotAvailable, InternalError);

    /*!
     * @if jp
     * 
     * @brief [CORBA interface] Organization λȤκ
     *
     * Υڥ졼 Organization λȤ롣
     *
     * @param organization_id  Organization ΰդ id
     * @return ڥ졼ɤ֤
     * @exception SDONotExists åȤSDO¸ߤʤ
     * @exception InvalidParameter 
     * @exception NotAvailable SDO¸ߤ뤬ʤ
     * @exception InternalError Ū顼ȯ
     * @else
     *
     * @brief [CORBA interface] Remove the reference of Organization 
     *
     * This operation removes the reference of an Organization object.
     *
     * @param organization_id Unique id of the organization to be removed.
     * @return If the operation was successfully completed.
     * @exception SDONotExists The target SDO does not exist.
     * @exception InvalidParameter The argument "organizationID" is null,
     *            or the object which is specified by argument "organizationID"
     *            does not exist.
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Boolean remove_organization(const char* organization_id)
      throw (InvalidParameter, NotAvailable, InternalError);

    /*!
     * @if jp
     * 
     * @brief [CORBA interface] ѥ᡼ΥꥹȤμ
     *
     * Υڥ졼 configuration parameter ΥꥹȤ֤
     * SDO ǽʥѥ᡼ʤжΥꥹȤ֤
     *
     * @return ħդѥ᡼Υꥹȡ
     * @exception SDONotExists åȤSDO¸ߤʤ
     * @exception NotAvailable SDO¸ߤ뤬ʤ
     * @exception InternalError Ū顼ȯ
     * @else
     *
     * @brief [CORBA interface] Getting a list of configuration parameter
     *
     * This operation returns a list of Parameters. An empty list is returned
     * if the SDO does not have any configurable parameter.
     *
     * @return The list with definitions of parameters characterizing the
     *          configuration.
     * @exception SDONotExists The target SDO does not exist.
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual ParameterList* get_configuration_parameters()
      throw (NotAvailable, InternalError);

     /*!
     * @if jp
     * 
     * @brief [CORBA interface] Configuration parameter ͤΥꥹȤμ
     *
     * Υڥ졼 configuration ѥ᡼֤ͤ
     *
     * @return Ƥ configuration ѥ᡼ͤΥꥹȡ
     * @exception SDONotExists åȤSDO¸ߤʤ
     * @exception NotAvailable SDO¸ߤ뤬ʤ
     * @exception InternalError Ū顼ȯ
     * @else
     *
     * @brief [CORBA interface] Getting value list of configuration parameter
     *
     * This operation returns all configuration parameters and their values.
     *
     * @return List of all configuration parameters and their values.
     * @exception SDONotExists The target SDO does not exist.
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual NVList* get_configuration_parameter_values()
      throw (NotAvailable, InternalError);

    /*!
     * @if jp
     * 
     * @brief [CORBA interface] Configuration parameter ͤμ
     *
     * Υڥ졼ϰ "name" ǻꤵ줿ѥ᡼֤ͤ
     *
     * @param name ͤ׵᤹ѥ᡼̾
     * @return ꤵ줿ѥ᡼͡
     * @exception SDONotExists åȤSDO¸ߤʤ
     * @exception NotAvailable SDO¸ߤ뤬ʤ
     * @exception InternalError Ū顼ȯ
     * @else
     *
     * @brief [CORBA interface] Getting value of configuration parameter
     *
     * This operation returns a value of parameter that is specified by
     * argument "name."
     *
     * @param Name of the parameter whose value is requested.
     * @return The value of the specified parameter.
     * @exception SDONotExists The target SDO does not exist.
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Any* get_configuration_parameter_value(const char* name)
      throw (InvalidParameter, NotAvailable, InternalError);

    /*!
     * @if jp
     * 
     * @brief [CORBA interface] Configuration ѥ᡼ѹ
     *
     * Υڥ졼 "name" ǻꤷѥ᡼ͤ "value" 
     * ѹ롣
     *
     * @param name ѹѥ᡼̾
     * @param value ѹѥ᡼͡
     * @return ڥ졼ɤ֤
     * @exception SDONotExists åȤSDO¸ߤʤ
     * @exception NotAvailable SDO¸ߤ뤬ʤ
     * @exception InternalError Ū顼ȯ
     * @else
     *
     * @brief [CORBA interface] Modify the parameter value
     *
     * This operation sets a parameter to a value that is specified by argument
     * "value." The parameter to be modified is specified by argument " name."
     *
     * @param name The name of parameter to be modified.
     * @param value New value of the specified parameter.
     * @return If the operation was successfully completed.
     * @exception SDONotExists The target SDO does not exist.
     * @exception InvalidParameter if arguments ("name" and/or "value") is
     *            null, or if the parameter that is specified by the argument
     *            "name" does not exist.
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Boolean set_configuration_parameter(const char* name,
						       const CORBA::Any& value)
      throw (InvalidParameter, NotAvailable, InternalError);

    /*!
     * @if jp
     * 
     * @brief [CORBA interface] ConfigurationSet ꥹȤμ 
     *
     * Υڥ졼 ConfigurationProfile  ConfigurationSet 
     * ꥹȤ֤ SDO  ConfigurationSet ʤжΥꥹȤ֤
     *
     * @return ݻƤ ConfigurationSet ΥꥹȤθ͡
     * @exception SDONotExists åȤSDO¸ߤʤ
     * @exception NotAvailable SDO¸ߤ뤬ʤ
     * @exception InternalError Ū顼ȯ
     * @else
     *
     * @brief [CORBA interface] Getting list of ConfigurationSet
     *
     * This operation returns a list of ConfigurationSets that the
     * ConfigurationProfile has. An empty list is returned if the SDO does not
     * have any ConfigurationSets.
     * This operation returns a list of all ConfigurationSets of the SDO.
     * If no predefined ConfigurationSets exist, then empty list is returned.
     *
     * @return The list of stored configuration with their current values.
     * @exception SDONotExists The target SDO does not exist.
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual ConfigurationSetList* get_configuration_sets()
      throw (NotAvailable, InternalError);

    /*!
     * @if jp
     * 
     * @brief [CORBA interface] ConfigurationSet μ
     *
     * Υڥ졼ϰǻꤵ줿 ConfigurationSet  ID ˴Ϣ
     * դ줿 ConfigurationSet ֤
     *
     * @param config_id ConfigurationSet μ̻ҡ
     * @return ˤꤵ줿 ConfigurationSet
     * @exception SDONotExists åȤSDO¸ߤʤ
     * @exception InvalidParameter "config_id"  null ꤵ줿
     *            ConfigurationSet ¸ߤʤ
     * @exception NotAvailable SDO¸ߤ뤬ʤ
     * @exception InternalError Ū顼ȯ
     * @else
     *
     * @brief [CORBA interface] Getting a ConfigurationSet
     *
     * This operation returns the ConfigurationSet specified by the parameter
     * configurationSetID.
     *
     * @param config_id Identifier of ConfigurationSet requested.
     * @return The configuration set specified by the parameter config_id.
     * @exception SDONotExists The target SDO does not exist.
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual ConfigurationSet* get_configuration_set(const char* config_id)
      throw (NotAvailable, InternalError);

    /*!
     * @if jp
     * 
     * @brief [CORBA interface] ConfigurationSet 򥻥åȤ
     *
     * Υڥ졼ϻꤵ줿 id  ConfigurationSet 򹹿롣
     *
     * @param configu_id ѹ ConfigurationSet  ID
     * @param configuration_set ѹ ConfigurationSet ΤΡ
     * @return ConfigurationSet ˹Ǥ true
     *         Ǥʤ false ֤
     * @exception InvalidParameter config_id  null  ConfigurationSet
     * @exception SDONotExists åȤSDO¸ߤʤ
     * @exception NotAvailable SDO¸ߤ뤬ʤ
     * @exception InternalError Ū顼ȯ
     * @else
     *
     * @brief [CORBA interface] Set ConfigurationSet
     *
     * This operation modifies the specified ConfigurationSet of an SDO.
     *
     *  ѥ᡼ο spec  IDL ǰۤʤ롪
     * @param configu_id The ID of ConfigurationSet to be modified.
     * @param configuration_set ConfigurationSet to be replaced.
     * @return A flag indicating if the ConfigurationSet was modified 
     *         successfully. "true" - The ConfigurationSet was modified
     *         successfully. "false" - The ConfigurationSet could not be
     *         modified successfully.
     * @exception InvalidParameter if the parameter 'configurationSetID' is
     *            null or if there is no ConfigurationSet stored with such id.
     *            This exception is also raised if one of the attributes
     *            defining ConfigurationSet is not valid.
     * @exception SDONotExists The target SDO does not exist.
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Boolean
    set_configuration_set_values(const char* config_id,
				 const ConfigurationSet& configuration_set)
      throw (InvalidParameter, NotAvailable, InternalError);

    /*!
     * @if jp
     * 
     * @brief [CORBA interface] ƥ֤ ConfigurationSet 
     *
     * Υڥ졼SDOθߥƥ֤ ConfigurationSet ֤
     * (⤷SDOθߤ꤬ͽ줿 ConfigurationSet ˤꤵ
     * ʤС)
     * ConfigurationSet ϰʲξˤϥƥ֤ǤϤʤΤȤߤʤ롣
     *
     * - ߤ꤬ͽ줿 ConfigurationSet ˤꥻåȤƤʤ
     * - SDO ꤬ƥ֤ˤʤäѹ줿
     * - SDO ꤹ ConfigurationSet ѹ줿
     * 
     * ξˤϡ ConfigurationSet ֤롣
     *
     * @return ߥƥ֤ ConfigurationSet
     * @exception SDONotExists åȤSDO¸ߤʤ
     * @exception NotAvailable SDO¸ߤ뤬ʤ
     * @exception InternalError Ū顼ȯ
     * @else
     *
     * @brief [CORBA interface] Get active ConfigurationSet
     *
     * This operation returns the current active ConfigurationSet of an
     * SDO (i.e., if the current configuration of the SDO was set using
     * predefined configuration set).
     * ConfigurationSet cannot be considered active if the:
     *
     * - current configuration of the SDO was not set using any predefined
     *   ConfigurationSet, or
     * - configuration of the SDO was changed after it has been active, or
     * - ConfigurationSet that was used to configure the SDO was modified.
     *
     * Empty ConfigurationSet is returned in these cases.
     *
     * @return The active ConfigurationSet.
     * @exception SDONotExists The target SDO does not exist.
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual ConfigurationSet* get_active_configuration_set()
      throw (NotAvailable, InternalError);

    /*!
     * @if jp
     * 
     * @brief [CORBA interface] ConfigurationSet ɲä
     *
     * ConfigurationProfile  ConfigurationSet ɲä륪ڥ졼
     *
     * @param configuration_set ɲä ConfigurationSet
     * @return ڥ졼ɤ
     * @exception SDONotExists åȤSDO¸ߤʤ
     * @exception NotAvailable SDO¸ߤ뤬ʤ
     * @exception InternalError Ū顼ȯ
     * @else
     *
     * @brief [CORBA interface] Add ConfigurationSet
     *
     * This operation adds a ConfigurationSet to the ConfigurationProfile.
     *
     * @param configuration_set The ConfigurationSet that is added.
     * @return If the operation was successfully completed.
     * @exception SDONotExists The target SDO does not exist.
     * @exception InvalidParameter If the argument "configurationSet" is null,
     *            or if one of the attributes defining "configurationSet" is
     *            invalid, or if the specified identifier of the configuration
     *            set already exists.
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Boolean
    add_configuration_set(const ConfigurationSet& configuration_set)
      throw (InvalidParameter, NotAvailable, InternalError);

    /*!
     * @if jp
     * 
     * @brief [CORBA interface] ConfigurationSet 
     *
     * ConfigurationProfile  ConfigurationSet 롣
     *
     * @param configu_id  ConfigurationSet  id
     * @return ڥ졼ɤ
     * @exception SDONotExists åȤSDO¸ߤʤ
     * @exception InvalidParameter  "configurationSetID"  null Ǥ롢
     *            ⤷ϡǻꤵ줿 ConfigurationSet ¸ߤʤ
     * @exception NotAvailable SDO¸ߤ뤬ʤ
     * @exception InternalError Ū顼ȯ
     * @else
     *
     * @brief [CORBA interface] Remove ConfigurationSet
     *
     * This operation removes a ConfigurationSet from the ConfigurationProfile.
     *
     * @param config_id The id of ConfigurationSet which is removed.
     * @return If the operation was successfully completed.
     * @exception SDONotExists The target SDO does not exist.
     * @exception InvalidParameter The arguments "configurationSetID" is null,
     *            or if the object specified by the argument
     *            "configurationSetID" does not exist.
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Boolean remove_configuration_set(const char* config_id)
      throw (InvalidParameter, NotAvailable, InternalError);

    /*!
     * @if jp
     * 
     * @brief [CORBA interface] ConfigurationSet Υƥֲ
     *
     * ConfigurationProfile ˳Ǽ줿 ConfigurationSet ΤĤ
     * ƥ֤ˤ롣
     * Υڥ졼 ConfigurationSet 򥢥ƥ֤ˤ롣
     * ʤSDO Υե졼󡦥ץѥƥγǼƤ
     * ConfigurationSet ˤꤵץѥƥͤѹ롣
     * ꤵ줿 ConfigurationSet ͤƥ֡ե졼
     * ˥ԡȤȤ̣롣
     *
     * @param config_id ƥֲ ConfigurationSet  id
     * @return ڥ졼ɤ
     * @exception SDONotExists åȤSDO¸ߤʤ
     * @exception InvalidParameter  "config_id"  null Ǥ롢⤷
     *            ǻꤵ줿 ConfigurationSet ¸ߤʤ
     * @exception NotAvailable SDO¸ߤ뤬ʤ
     * @exception InternalError Ū顼ȯ
     * @else
     *
     * @brief [CORBA interface] Activate ConfigurationSet
     *
     * This operation activates one of the stored ConfigurationSets in the
     * ConfigurationProfile.
     * This operation activates the specified stored ConfigurationSets.
     * This means that the configuration properties of the SDO are changed as
     * the values of these properties specified in the stored ConfigurationSet.
     * In other words, values of the specified ConfigurationSet are now copied
     * to the active configuration.
     *
     * @param Identifier of ConfigurationSet to be activated.
     * @return If the operation was successfully completed.
     * @exception SDONotExists The target SDO does not exist.
     * @exception InvalidParameter if the argument ("configID") is null or
     *            there is no configuration set with identifier specified by
     *            the argument.
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual CORBA::Boolean activate_configuration_set(const char* config_id)
      throw (InvalidParameter, NotAvailable, InternalError);

    // end of CORBA interface definition
    //============================================================

    Configuration_ptr getObjRef();

    const DeviceProfile getDeviceProfile();

    const ServiceProfileList getServiceProfiles();

    const ServiceProfile getServiceProfile(const char* id);
    
    const OrganizationList getOrganizations();

  protected:
    const std::string getUUID() const;

    Configuration_var m_objref;

    typedef ACE_Guard<ACE_Thread_Mutex> Guard;
    /*!
     * @if jp
     * @brief Lock դ SDO DeviceProfile
     * @else
     * @brief SDO DeviceProfile with mutex lock
     * @endif
     */
    DeviceProfile m_deviceProfile;
    ACE_Thread_Mutex m_dprofile_mutex;

    /*!
     * @if jp
     * @brief SDO ServiceProfileList
     * @else
     * @brief SDO ServiceProfileList
     * @endif
     */
    ServiceProfileList m_serviceProfiles;
    ACE_Thread_Mutex m_sprofile_mutex;

    /*!
     * @if jp
     * @brief SDO ParameterList
     * @else
     * @brief SDO ParameterList
     * @endif
     */
    /*
    struct Parameter
    {
      string         name;
      TypeCode  type;
      AllowedValues allowed_values;
    };
    */
    ParameterList m_parameters;
    ACE_Thread_Mutex m_params_mutex;

    /*!
     * @if jp
     * @brief SDO ConfigurationSetList
     * @else
     * @brief SDO ConfigurationSetList
     * @endif
     */
    /*
      struct ConfigurationSet
      {
      string id;
      string description;
      NVList configuration_data;
      };
    */
    RTC::ConfigAdmin& m_configsets;
    ACE_Thread_Mutex m_config_mutex;

    /*!
     * @if jp
     * @brief SDO OrganizationList
     * @else
     * @brief SDO OrganizationList
     * @endif
     */
    OrganizationList m_organizations;
    ACE_Thread_Mutex m_org_mutex;

    /*!
     * @if jp
     * @brief ƥ֤ ConfigurationSet
     * @else
     * @brief Active ConfigurationSet id
     * @endif
     */

    struct nv_name
    {
      nv_name(const char* name) : m_name(name) {};
      bool operator()(const NameValue& nv)
      {
	return m_name == std::string(nv.name);
      }
      std::string m_name;
    };


    // functor for ServiceProfile
    struct service_id
    {
      service_id(const char* id) : m_id(id) {};
      bool operator()(const ServiceProfile& s)
      {
	std::string id(s.id);
	return m_id == id;
      }
      const std::string m_id;
    };

    // functor for Organization
    struct org_id
    {
      org_id(const char* id) : m_id(id) {};
      bool operator()(const Organization_ptr& o)
      {
	std::string id(o->get_organization_id());
	return m_id == id;
      }
      const std::string m_id;
    };
    
    // functor for ConfigurationSet
    struct config_id
    {
      config_id(const char* id) :  m_id(id) {};
      bool operator()(const ConfigurationSet& c)
      {
	std::string id(c.id);
	return m_id == id;
      }
      const std::string m_id;
    };

  };

}; // namespace SDOPackage

#endif // SdoConfiguration_h
